package java.math;

import sun.misc.Unsafe;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.io.ObjectInputStream;
import java.util.Arrays;

public class BigDecimal extends Number implements Comparable<BigDecimal>
{
    private final BigInteger intVal;
    private final int scale;
    private transient int precision;
    private transient String stringCache;
    static final long INFLATED = Long.MIN_VALUE;
    private static final BigInteger INFLATED_BIGINT;
    private final transient long intCompact;
    private static final int MAX_COMPACT_DIGITS = 18;
    private static final long serialVersionUID = 6108874887143696463L;
    private static final ThreadLocal<BigDecimal.StringBuilderHelper> threadLocalStringBuilderHelper;
    private static final BigDecimal[] zeroThroughTen;
    private static final BigDecimal[] ZERO_SCALED_BY;
    private static final long HALF_LONG_MAX_VALUE = 4611686018427387903L;
    private static final long HALF_LONG_MIN_VALUE = -4611686018427387904L;
    public static final BigDecimal ZERO;
    public static final BigDecimal ONE;
    public static final BigDecimal TEN;
    public static final int ROUND_UP = 0;
    public static final int ROUND_DOWN = 1;
    public static final int ROUND_CEILING = 2;
    public static final int ROUND_FLOOR = 3;
    public static final int ROUND_HALF_UP = 4;
    public static final int ROUND_HALF_DOWN = 5;
    public static final int ROUND_HALF_EVEN = 6;
    public static final int ROUND_UNNECESSARY = 7;
    private static final double[] double10pow;
    private static final float[] float10pow;
    private static final long[] LONG_TEN_POWERS_TABLE;
    private static volatile BigInteger[] BIG_TEN_POWERS_TABLE;
    private static final int BIG_TEN_POWERS_TABLE_INITLEN;
    private static final int BIG_TEN_POWERS_TABLE_MAX;
    private static final long[] THRESHOLDS_TABLE;
    private static final long DIV_NUM_BASE = 4294967296L;
    private static final long[][] LONGLONG_TEN_POWERS_TABLE;
    
    BigDecimal(BigInteger intVal, long intCompact, int scale, int precision) {
        this.scale = scale;
        this.precision = precision;
        this.intCompact = intCompact;
        this.intVal = intVal;
    }
    
    public BigDecimal(char[] array, int n, int n2) {
        this(array, n, n2, MathContext.UNLIMITED);
    }
    
    public BigDecimal(char[] array, int n, int i, MathContext mathContext) {
        if ((array.length | i | n) < 0 || i > array.length - n) {
            throw new NumberFormatException("Bad offset or len arguments for char[] input.");
        }
        int precision = 0;
        int scale = 0;
        long n2 = 0L;
        Object divideAndRoundByTenPow = null;
        long intCompact;
        try {
            boolean n3 = false;
            if (array[n] == '-') {
                n3 = true;
                ++n;
                --i;
            }
            else if (array[n] == '+') {
                ++n;
                --i;
            }
            int n4 = 0;
            long n5 = 0L;
            int n6 = i <= 18;
            int n7 = 0;
            if (!(n6 == 0)) {
                while (i > 0) {
                    int n9 = array[n];
                    if (n9 == '0') {
                        if (precision == 0) {
                            precision = 1;
                        }
                        else if (n2 != 0L) {
                            n2 *= 10L;
                            ++precision;
                        }
                        if (n4 != 0) {
                            ++scale;
                        }
                    }
                    else if (n9 >= '1' && n9 <= '9') {
                        int n10 = n9 - 48;
                        if (precision != 1 || n2 != 0L) {
                            ++precision;
                        }
                        n2 = n2 * 10L + (long)n10;
                        if (n4 != 0) {
                            ++scale;
                        }
                    }
                    else if (n9 == '.') {
                        if (n4 != 0) {
                            throw new NumberFormatException();
                        }
                        n4 = 1;
                    }
                    else if (Character.isDigit((char)n9)) {
                        int digit = Character.digit((char)n9, 10);
                        if (digit == 0) {
                            if (precision == 0) {
                                precision = 1;
                            }
                            else if (n2 != 0L) {
                                n2 *= 10L;
                                ++precision;
                            }
                        }
                        else {
                            if (precision != 1 || n2 != 0L) {
                                ++precision;
                            }
                            n2 = n2 * 10L + (long)digit;
                        }
                        if (n4 != 0) {
                            ++scale;
                        }
                    }
                    else if (n9 == 'e' || n9 == 'E') {
                        n5 = parseExp(array, n, i);
                        if ((long)(int)n5 != n5) {
                            throw new NumberFormatException();
                        }
                        break;
                    }
                    else {
                        throw new NumberFormatException();
                    }
                    ++n;
                    --i;
                }
                if (precision == 0) {
                    throw new NumberFormatException();
                }
                if (n5 != 0L) {
                    scale = this.adjustScale(scale, n5);
                }
                intCompact = (n3 ? (-n2) : n2);
                int precision3 = mathContext.precision;
                int l = precision - precision3;
                if (precision3 > 0 && l > 0) {
                    while (l > 0) {
                        scale = checkScaleNonZero((long)scale - (long)l);
                        intCompact = divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[l], mathContext.roundingMode.oldMode);
                        precision = longDigitLength(intCompact);
                        l = precision - precision3;
                    }
                }
            }
            else {
                char[] array2 = new char[i];
                while (i > 0) {
                    int n8 = array[n];
                    if ((n8 >= '0' && n8 <= '9') || Character.isDigit((char)n8)) {
                        if (n8 == '0' || Character.digit((char)n8, 10) == 0) {
                            if (precision == 0) {
                                array2[n7] = n8;
                                precision = 1;
                            }
                            else if (n7 != 0) {
                                array2[n7++] = n8;
                                ++precision;
                            }
                        }
                        else {
                            if (precision != 1 || n7 != 0) {
                                ++precision;
                            }
                            array2[n7++] = n8;
                        }
                        if (n4 != 0) {
                            ++scale;
                        }
                    }
                    else if (n8 == '.') {
                        if (n4 != 0) {
                            throw new NumberFormatException();
                        }
                        n4 = 1;
                    }
                    else {
                        if (n8 != 'e' && n8 != 'E') {
                            throw new NumberFormatException();
                        }
                        n5 = parseExp(array, n, i);
                        if ((long)(int)n5 != n5) {
                            throw new NumberFormatException();
                        }
                        break;
                    }
                    ++n;
                    --i;
                }
                if (precision == 0) {
                    throw new NumberFormatException();
                }
                if (n5 != 0L) {
                    scale = this.adjustScale(scale, n5);
                }
                divideAndRoundByTenPow = new BigInteger(array2, n3 ? -1 : 1, precision);
                intCompact = compactValFor(divideAndRoundByTenPow);
                int precision2 = mathContext.precision;
                if (precision2 > 0 && precision > precision2) {
                    if (intCompact == Long.MIN_VALUE) {
                        int j = precision - precision2;
                        while (j > 0) {
                            scale = checkScaleNonZero((long)scale - (long)j);
                            divideAndRoundByTenPow = divideAndRoundByTenPow(divideAndRoundByTenPow, j, mathContext.roundingMode.oldMode);
                            intCompact = compactValFor(divideAndRoundByTenPow);
                            if (intCompact != Long.MIN_VALUE) {
                                precision = longDigitLength(intCompact);
                                break;
                            }
                            precision = bigDigitLength(divideAndRoundByTenPow);
                            j = precision - precision2;
                        }
                    }
                    if (intCompact != Long.MIN_VALUE) {
                        for (int k = precision - precision2; k > 0; k = precision - precision2) {
                            scale = checkScaleNonZero((long)scale - (long)k);
                            intCompact = divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[k], mathContext.roundingMode.oldMode);
                            precision = longDigitLength(intCompact);
                        }
                        divideAndRoundByTenPow = null;
                    }
                }
            }
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new NumberFormatException();
        }
        catch (NegativeArraySizeException ex2) {
            throw new NumberFormatException();
        }
        this.scale = scale;
        this.precision = precision;
        this.intCompact = intCompact;
        this.intVal = divideAndRoundByTenPow;
        return;
    }
    
    private int adjustScale(int n, long n2) {
        long n3 = (long)n - n2;
        if (n3 > 2147483647L || n3 < -2147483648L) {
            throw new NumberFormatException("Scale out of range.");
        }
        n = (int)n3;
        return n;
    }
    
    private static long parseExp(char[] array, int n, int n2) {
        long n3 = 0L;
        ++n;
        int n4 = array[n];
        --n2;
        int n5 = n4 == '-';
        if (n5 || n4 == '+') {
            ++n;
            n4 = array[n];
            --n2;
        }
        if (n2 <= 0) {
            throw new NumberFormatException();
        }
        while (n2 > 10 && (n4 == '0' || Character.digit((char)n4, 10) == 0)) {
            ++n;
            n4 = array[n];
            --n2;
        }
        if (n2 > 10) {
            throw new NumberFormatException();
        }
        while (true) {
            int digit;
            if (n4 >= '0' && n4 <= '9') {
                digit = n4 - 48;
            }
            else {
                digit = Character.digit((char)n4, 10);
                if (digit < 0) {
                    throw new NumberFormatException();
                }
            }
            n3 = n3 * 10L + (long)digit;
            if (n2 == 1) {
                if (!(n5 == 0)) {
                    n3 = -n3;
                }
                return n3;
            }
            else {
                ++n;
                n4 = array[n];
                --n2;
            }
        }
    }
    
    public BigDecimal(char[] array) {
        this(array, 0, array.length);
    }
    
    public BigDecimal(char[] array, MathContext mathContext) {
        this(array, 0, array.length, mathContext);
    }
    
    public BigDecimal(String s) {
        this(s.toCharArray(), 0, s.length());
    }
    
    public BigDecimal(String s, MathContext mathContext) {
        this(s.toCharArray(), 0, s.length(), mathContext);
    }
    
    public BigDecimal(double n) {
        this(n, MathContext.UNLIMITED);
    }
    
    public BigDecimal(double value, MathContext mathContext) {
        if (Double.isInfinite(value) || Double.isNaN(value)) {
            throw new NumberFormatException("Infinite or NaN");
        }
        long doubleToLongBits = Double.doubleToLongBits(value);
        int n = (doubleToLongBits >> 63 == 0L) ? 1 : -1;
        int n2 = (int)(doubleToLongBits >> 52 & 0x7FFL);
        long n3 = (n2 == 0) ? ((doubleToLongBits & 0xFFFFFFFFFFFFFL) << 1) : ((doubleToLongBits & 0xFFFFFFFFFFFFFL) | 0x10000000000000L);
        n2 -= 1075;
        if (n3 == 0L) {
            this.intVal = BigInteger.ZERO;
            this.scale = 0;
            this.intCompact = 0L;
            this.precision = 1;
            return;
        }
        while ((n3 & 0x1L) == 0x0L) {
            n3 >>= 1;
            ++n2;
        }
        int scale = 0;
        long intCompact = (long)n * n3;
        BigInteger intVal;
        if (n2 == 0) {
            intVal = ((intCompact == Long.MIN_VALUE) ? BigDecimal.INFLATED_BIGINT : null);
        }
        else {
            if (n2 < 0) {
                intVal = BigInteger.valueOf(5L).pow(-n2).multiply(intCompact);
                scale = -n2;
            }
            else {
                intVal = BigInteger.valueOf(2L).pow(n2).multiply(intCompact);
            }
            intCompact = compactValFor(intVal);
        }
        int precision = 0;
        int precision2 = mathContext.precision;
        if (precision2 > 0) {
            int oldMode = mathContext.roundingMode.oldMode;
            if (intCompact == Long.MIN_VALUE) {
                precision = bigDigitLength(intVal);
                int i = precision - precision2;
                while (i > 0) {
                    scale = checkScaleNonZero((long)scale - (long)i);
                    intVal = divideAndRoundByTenPow(intVal, i, oldMode);
                    intCompact = compactValFor(intVal);
                    if (intCompact != Long.MIN_VALUE) {
                        break;
                    }
                    precision = bigDigitLength(intVal);
                    i = precision - precision2;
                }
            }
            if (intCompact != Long.MIN_VALUE) {
                precision = longDigitLength(intCompact);
                for (int j = precision - precision2; j > 0; j = precision - precision2) {
                    scale = checkScaleNonZero((long)scale - (long)j);
                    intCompact = divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[j], mathContext.roundingMode.oldMode);
                    precision = longDigitLength(intCompact);
                }
                intVal = null;
            }
        }
        this.intVal = intVal;
        this.intCompact = intCompact;
        this.scale = scale;
        this.precision = precision;
        return;
    }
    
    public BigDecimal(BigInteger intVal) {
        this.scale = 0;
        this.intVal = intVal;
        this.intCompact = compactValFor(intVal);
    }
    
    public BigDecimal(BigInteger bigInteger, MathContext mathContext) {
        this(bigInteger, 0, mathContext);
    }
    
    public BigDecimal(BigInteger intVal, int scale) {
        this.intVal = intVal;
        this.intCompact = compactValFor(intVal);
        this.scale = scale;
    }
    
    public BigDecimal(BigInteger divideAndRoundByTenPow, int scale, MathContext mathContext) {
        long intCompact = compactValFor(divideAndRoundByTenPow);
        int precision = mathContext.precision;
        int precision2 = 0;
        if (precision > 0) {
            int oldMode = mathContext.roundingMode.oldMode;
            if (intCompact == Long.MIN_VALUE) {
                precision2 = bigDigitLength(divideAndRoundByTenPow);
                int i = precision2 - precision;
                while (i > 0) {
                    scale = checkScaleNonZero((long)scale - (long)i);
                    divideAndRoundByTenPow = divideAndRoundByTenPow(divideAndRoundByTenPow, i, oldMode);
                    intCompact = compactValFor(divideAndRoundByTenPow);
                    if (intCompact != Long.MIN_VALUE) {
                        break;
                    }
                    precision2 = bigDigitLength(divideAndRoundByTenPow);
                    i = precision2 - precision;
                }
            }
            if (intCompact != Long.MIN_VALUE) {
                precision2 = longDigitLength(intCompact);
                for (int j = precision2 - precision; j > 0; j = precision2 - precision) {
                    scale = checkScaleNonZero((long)scale - (long)j);
                    intCompact = divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[j], oldMode);
                    precision2 = longDigitLength(intCompact);
                }
                divideAndRoundByTenPow = null;
            }
        }
        this.intVal = divideAndRoundByTenPow;
        this.intCompact = intCompact;
        this.scale = scale;
        this.precision = precision2;
    }
    
    public BigDecimal(int n) {
        this.intCompact = (long)n;
        this.scale = 0;
        this.intVal = null;
    }
    
    public BigDecimal(int n, MathContext mathContext) {
        int precision = mathContext.precision;
        long divideAndRound = (long)n;
        int checkScaleNonZero = 0;
        int precision2 = 0;
        if (precision > 0) {
            precision2 = longDigitLength(divideAndRound);
            for (int i = precision2 - precision; i > 0; i = precision2 - precision) {
                checkScaleNonZero = checkScaleNonZero((long)checkScaleNonZero - (long)i);
                divideAndRound = divideAndRound(divideAndRound, BigDecimal.LONG_TEN_POWERS_TABLE[i], mathContext.roundingMode.oldMode);
                precision2 = longDigitLength(divideAndRound);
            }
        }
        this.intVal = null;
        this.intCompact = divideAndRound;
        this.scale = checkScaleNonZero;
        this.precision = precision2;
    }
    
    public BigDecimal(long intCompact) {
        this.intCompact = intCompact;
        this.intVal = ((intCompact == Long.MIN_VALUE) ? BigDecimal.INFLATED_BIGINT : null);
        this.scale = 0;
    }
    
    public BigDecimal(long intCompact, MathContext mathContext) {
        int precision = mathContext.precision;
        int oldMode = mathContext.roundingMode.oldMode;
        int precision2 = 0;
        int scale = 0;
        BigInteger divideAndRoundByTenPow = (intCompact == Long.MIN_VALUE) ? BigDecimal.INFLATED_BIGINT : null;
        if (precision > 0) {
            if (intCompact == Long.MIN_VALUE) {
                precision2 = 19;
                int i = precision2 - precision;
                while (i > 0) {
                    scale = checkScaleNonZero((long)scale - (long)i);
                    divideAndRoundByTenPow = divideAndRoundByTenPow(divideAndRoundByTenPow, i, oldMode);
                    intCompact = compactValFor(divideAndRoundByTenPow);
                    if (intCompact != Long.MIN_VALUE) {
                        break;
                    }
                    precision2 = bigDigitLength(divideAndRoundByTenPow);
                    i = precision2 - precision;
                }
            }
            if (intCompact != Long.MIN_VALUE) {
                precision2 = longDigitLength(intCompact);
                for (int j = precision2 - precision; j > 0; j = precision2 - precision) {
                    scale = checkScaleNonZero((long)scale - (long)j);
                    intCompact = divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[j], mathContext.roundingMode.oldMode);
                    precision2 = longDigitLength(intCompact);
                }
                divideAndRoundByTenPow = null;
            }
        }
        this.intVal = divideAndRoundByTenPow;
        this.intCompact = intCompact;
        this.scale = scale;
        this.precision = precision2;
    }
    
    public static BigDecimal valueOf(long n, int n2) {
        if (n2 == 0) {
            return valueOf(n);
        }
        if (n == 0L) {
            return zeroValueOf(n2);
        }
        return new BigDecimal((n == Long.MIN_VALUE) ? BigDecimal.INFLATED_BIGINT : null, n, n2, 0);
    }
    
    public static BigDecimal valueOf(long n) {
        if (n >= 0L && n < (long)BigDecimal.zeroThroughTen.length) {
            return BigDecimal.zeroThroughTen[(int)n];
        }
        if (n != Long.MIN_VALUE) {
            return new BigDecimal(null, n, 0, 0);
        }
        return new BigDecimal(BigDecimal.INFLATED_BIGINT, n, 0, 0);
    }
    
    static BigDecimal valueOf(long n, int n2, int n3) {
        if (n2 == 0 && n >= 0L && n < (long)BigDecimal.zeroThroughTen.length) {
            return BigDecimal.zeroThroughTen[(int)n];
        }
        if (n == 0L) {
            return zeroValueOf(n2);
        }
        return new BigDecimal((n == Long.MIN_VALUE) ? BigDecimal.INFLATED_BIGINT : null, n, n2, n3);
    }
    
    static BigDecimal valueOf(BigInteger bigInteger, int n, int n2) {
        long compactVal = compactValFor(bigInteger);
        if (compactVal == 0L) {
            return zeroValueOf(n);
        }
        if (n == 0 && compactVal >= 0L && compactVal < (long)BigDecimal.zeroThroughTen.length) {
            return BigDecimal.zeroThroughTen[(int)compactVal];
        }
        return new BigDecimal(bigInteger, compactVal, n, n2);
    }
    
    static BigDecimal zeroValueOf(int n) {
        if (n >= 0 && n < BigDecimal.ZERO_SCALED_BY.length) {
            return BigDecimal.ZERO_SCALED_BY[n];
        }
        return new BigDecimal(BigInteger.ZERO, 0L, n, 1);
    }
    
    public static BigDecimal valueOf(double d) {
        return new BigDecimal(Double.toString(d));
    }
    
    public BigDecimal add(BigDecimal bigDecimal) {
        if (this.intCompact != Long.MIN_VALUE) {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return add(this.intCompact, this.scale, bigDecimal.intCompact, bigDecimal.scale);
            }
            return add(this.intCompact, this.scale, bigDecimal.intVal, bigDecimal.scale);
        }
        else {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return add(bigDecimal.intCompact, bigDecimal.scale, this.intVal, this.scale);
            }
            return add(this.intVal, this.scale, bigDecimal.intVal, bigDecimal.scale);
        }
    }
    
    public BigDecimal add(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.add(bigDecimal);
        }
        int n = this.signum() == 0;
        int n2 = bigDecimal.signum() == 0;
        if (n || n2) {
            int max = Math.max(this.scale(), bigDecimal.scale());
            if (n && n2) {
                return zeroValueOf(max);
            }
            BigDecimal bigDecimal2 = n ? doRound(bigDecimal, mathContext) : doRound(this, mathContext);
            if (bigDecimal2.scale() == max) {
                return bigDecimal2;
            }
            if (bigDecimal2.scale() > max) {
                return stripZerosToMatchScale(bigDecimal2.intVal, bigDecimal2.intCompact, bigDecimal2.scale, max);
            }
            int n4 = mathContext.precision - bigDecimal2.precision();
            if (n4 >= max - bigDecimal2.scale()) {
                return bigDecimal2.setScale(max);
            }
            return bigDecimal2.setScale(bigDecimal2.scale() + n4);
        }
        else {
            long n3 = (long)this.scale - (long)bigDecimal.scale;
            if (n3 != 0L) {
                BigDecimal[] preAlign = this.preAlign(this, bigDecimal, n3, mathContext);
                matchScale(preAlign);
                this = preAlign[0];
                bigDecimal = preAlign[1];
            }
            return doRound(this.inflated().add(bigDecimal.inflated()), this.scale, mathContext);
        }
    }
    
    private BigDecimal[] preAlign(BigDecimal bigDecimal, BigDecimal bigDecimal2, long n, MathContext mathContext) {
        assert n != 0L;
        BigDecimal value;
        if (n < 0L) {
            value = bigDecimal2;
        }
        else {
            bigDecimal = (value = bigDecimal2);
        }
        long b = (long)bigDecimal.scale - (long)bigDecimal.precision() + (long)mathContext.precision;
        long n2 = (long)value.scale - (long)value.precision() + 1L;
        if (n2 > (long)(bigDecimal.scale + 2) && n2 > b + 2L) {
            value = valueOf((long)value.signum(), this.checkScale(Math.max((long)bigDecimal.scale, b) + 3L));
        }
        return new BigDecimal[] { bigDecimal, value };
    }
    
    public BigDecimal subtract(BigDecimal bigDecimal) {
        if (this.intCompact != Long.MIN_VALUE) {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return add(this.intCompact, this.scale, -bigDecimal.intCompact, bigDecimal.scale);
            }
            return add(this.intCompact, this.scale, bigDecimal.intVal.negate(), bigDecimal.scale);
        }
        else {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return add(-bigDecimal.intCompact, bigDecimal.scale, this.intVal, this.scale);
            }
            return add(this.intVal, this.scale, bigDecimal.intVal.negate(), bigDecimal.scale);
        }
    }
    
    public BigDecimal subtract(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.subtract(bigDecimal);
        }
        return this.add(bigDecimal.negate(), mathContext);
    }
    
    public BigDecimal multiply(BigDecimal bigDecimal) {
        int checkScale = this.checkScale((long)this.scale + (long)bigDecimal.scale);
        if (this.intCompact != Long.MIN_VALUE) {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return multiply(this.intCompact, bigDecimal.intCompact, checkScale);
            }
            return multiply(this.intCompact, bigDecimal.intVal, checkScale);
        }
        else {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return multiply(bigDecimal.intCompact, this.intVal, checkScale);
            }
            return multiply(this.intVal, bigDecimal.intVal, checkScale);
        }
    }
    
    public BigDecimal multiply(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.multiply(bigDecimal);
        }
        int checkScale = this.checkScale((long)this.scale + (long)bigDecimal.scale);
        if (this.intCompact != Long.MIN_VALUE) {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return multiplyAndRound(this.intCompact, bigDecimal.intCompact, checkScale, mathContext);
            }
            return multiplyAndRound(this.intCompact, bigDecimal.intVal, checkScale, mathContext);
        }
        else {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return multiplyAndRound(bigDecimal.intCompact, this.intVal, checkScale, mathContext);
            }
            return multiplyAndRound(this.intVal, bigDecimal.intVal, checkScale, mathContext);
        }
    }
    
    public BigDecimal divide(BigDecimal bigDecimal, int n, int n2) {
        if (n2 < 0 || n2 > 7) {
            throw new IllegalArgumentException("Invalid rounding mode");
        }
        if (this.intCompact != Long.MIN_VALUE) {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return divide(this.intCompact, this.scale, bigDecimal.intCompact, bigDecimal.scale, n, n2);
            }
            return divide(this.intCompact, this.scale, bigDecimal.intVal, bigDecimal.scale, n, n2);
        }
        else {
            if (bigDecimal.intCompact != Long.MIN_VALUE) {
                return divide(this.intVal, this.scale, bigDecimal.intCompact, bigDecimal.scale, n, n2);
            }
            return divide(this.intVal, this.scale, bigDecimal.intVal, bigDecimal.scale, n, n2);
        }
    }
    
    public BigDecimal divide(BigDecimal bigDecimal, int n, RoundingMode roundingMode) {
        return this.divide(bigDecimal, n, roundingMode.oldMode);
    }
    
    public BigDecimal divide(BigDecimal bigDecimal, int n) {
        return this.divide(bigDecimal, this.scale, n);
    }
    
    public BigDecimal divide(BigDecimal bigDecimal, RoundingMode roundingMode) {
        return this.divide(bigDecimal, this.scale, roundingMode.oldMode);
    }
    
    public BigDecimal divide(BigDecimal bigDecimal) {
        if (bigDecimal.signum() == 0) {
            if (this.signum() == 0) {
                throw new ArithmeticException("Division undefined");
            }
            throw new ArithmeticException("Division by zero");
        }
        else {
            int saturateLong = saturateLong((long)this.scale - (long)bigDecimal.scale);
            if (this.signum() == 0) {
                return zeroValueOf(saturateLong);
            }
            MathContext mathContext = new MathContext((int)Math.min((long)this.precision() + (long)Math.ceil(10.0 * (double)bigDecimal.precision() / 3.0), 2147483647L), RoundingMode.UNNECESSARY);
            BigDecimal divide;
            try {
                divide = this.divide(bigDecimal, mathContext);
            }
            catch (ArithmeticException ex) {
                throw new ArithmeticException("Non-terminating decimal expansion; no exact representable decimal result.");
            }
            if (saturateLong > divide.scale()) {
                return divide.setScale(saturateLong, 7);
            }
            return divide;
        }
    }
    
    public BigDecimal divide(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.divide(bigDecimal);
        }
        long n = (long)this.scale - (long)bigDecimal.scale;
        if (bigDecimal.signum() == 0) {
            if (this.signum() == 0) {
                throw new ArithmeticException("Division undefined");
            }
            throw new ArithmeticException("Division by zero");
        }
        else {
            if (this.signum() == 0) {
                return zeroValueOf(saturateLong(n));
            }
            int precision = this.precision();
            int precision2 = bigDecimal.precision();
            if (this.intCompact != Long.MIN_VALUE) {
                if (bigDecimal.intCompact != Long.MIN_VALUE) {
                    return divide(this.intCompact, precision, bigDecimal.intCompact, precision2, n, mathContext);
                }
                return divide(this.intCompact, precision, bigDecimal.intVal, precision2, n, mathContext);
            }
            else {
                if (bigDecimal.intCompact != Long.MIN_VALUE) {
                    return divide(this.intVal, precision, bigDecimal.intCompact, precision2, n, mathContext);
                }
                return divide(this.intVal, precision, bigDecimal.intVal, precision2, n, mathContext);
            }
        }
    }
    
    public BigDecimal divideToIntegralValue(BigDecimal bigDecimal) {
        int saturateLong = saturateLong((long)this.scale - (long)bigDecimal.scale);
        if (this.compareMagnitude(bigDecimal) < 0) {
            return zeroValueOf(saturateLong);
        }
        if (this.signum() == 0 && bigDecimal.signum() != 0) {
            return this.setScale(saturateLong, 7);
        }
        BigDecimal bigDecimal2 = this.divide(bigDecimal, new MathContext((int)Math.min((long)this.precision() + (long)Math.ceil(10.0 * (double)bigDecimal.precision() / 3.0) + Math.abs((long)this.scale() - (long)bigDecimal.scale()) + 2L, 2147483647L), RoundingMode.DOWN));
        if (bigDecimal2.scale > 0) {
            BigDecimal setScale = bigDecimal2.setScale(0, RoundingMode.DOWN);
            bigDecimal2 = stripZerosToMatchScale(setScale.intVal, setScale.intCompact, setScale.scale, saturateLong);
        }
        if (bigDecimal2.scale < saturateLong) {
            bigDecimal2 = bigDecimal2.setScale(saturateLong, 7);
            return bigDecimal2;
        }
        return bigDecimal2;
    }
    
    public BigDecimal divideToIntegralValue(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0 || this.compareMagnitude(bigDecimal) < 0) {
            return this.divideToIntegralValue(bigDecimal);
        }
        int saturateLong = saturateLong((long)this.scale - (long)bigDecimal.scale);
        BigDecimal bigDecimal2 = this.divide(bigDecimal, new MathContext(mathContext.precision, RoundingMode.DOWN));
        if (bigDecimal2.scale() < 0) {
            if (this.subtract(bigDecimal2.multiply(bigDecimal)).compareMagnitude(bigDecimal) >= 0) {
                throw new ArithmeticException("Division impossible");
            }
        }
        else if (bigDecimal2.scale() > 0) {
            bigDecimal2 = bigDecimal2.setScale(0, RoundingMode.DOWN);
        }
        int a;
        if (saturateLong > bigDecimal2.scale() && (a = mathContext.precision - bigDecimal2.precision()) > 0) {
            return bigDecimal2.setScale(bigDecimal2.scale() + Math.min(a, saturateLong - bigDecimal2.scale));
        }
        return stripZerosToMatchScale(bigDecimal2.intVal, bigDecimal2.intCompact, bigDecimal2.scale, saturateLong);
    }
    
    public BigDecimal remainder(BigDecimal bigDecimal) {
        return this.divideAndRemainder(bigDecimal)[1];
    }
    
    public BigDecimal remainder(BigDecimal bigDecimal, MathContext mathContext) {
        return this.divideAndRemainder(bigDecimal, mathContext)[1];
    }
    
    public BigDecimal[] divideAndRemainder(BigDecimal bigDecimal) {
        BigDecimal[] array = { this.divideToIntegralValue(bigDecimal), null };
        array[1] = this.subtract(array[0].multiply(bigDecimal));
        return array;
    }
    
    public BigDecimal[] divideAndRemainder(BigDecimal bigDecimal, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.divideAndRemainder(bigDecimal);
        }
        BigDecimal[] array = new BigDecimal[2];
        array[0] = this.divideToIntegralValue(bigDecimal, mathContext);
        array[1] = this.subtract(array[0].multiply(bigDecimal));
        return array;
    }
    
    public BigDecimal pow(int n) {
        if (n < 0 || n > 999999999) {
            throw new ArithmeticException("Invalid operation");
        }
        return new BigDecimal(this.inflated().pow(n), this.checkScale((long)this.scale * (long)n));
    }
    
    public BigDecimal pow(int a, MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this.pow(a);
        }
        if (a < -999999999 || a > 999999999) {
            throw new ArithmeticException("Invalid operation");
        }
        if (a == 0) {
            return BigDecimal.ONE;
        }
        int abs = Math.abs(a);
        if (mathContext.precision > 0) {
            int longDigitLength = longDigitLength((long)abs);
            if (longDigitLength > mathContext.precision) {
                throw new ArithmeticException("Invalid operation");
            }
            mathContext = new MathContext(mathContext.precision + longDigitLength + 1, mathContext.roundingMode);
        }
        BigDecimal bigDecimal = BigDecimal.ONE;
        boolean n = false;
        int n2 = 1;
        while (true) {
            abs += abs;
            if (abs < 0) {
                n = true;
                bigDecimal = bigDecimal.multiply(this, mathContext);
            }
            if (n2 == 31) {
                break;
            }
            if (!(n == 0)) {
                bigDecimal = bigDecimal.multiply(bigDecimal, mathContext);
            }
            ++n2;
        }
        if (a < 0) {
            bigDecimal = BigDecimal.ONE.divide(bigDecimal, mathContext);
        }
        return doRound(bigDecimal, mathContext);
    }
    
    public BigDecimal abs() {
        return (this.signum() < 0) ? this.negate() : this;
    }
    
    public BigDecimal abs(MathContext mathContext) {
        return (this.signum() < 0) ? this.negate(mathContext) : this.plus(mathContext);
    }
    
    public BigDecimal negate() {
        if (this.intCompact == Long.MIN_VALUE) {
            return new BigDecimal(this.intVal.negate(), Long.MIN_VALUE, this.scale, this.precision);
        }
        return valueOf(-this.intCompact, this.scale, this.precision);
    }
    
    public BigDecimal negate(MathContext mathContext) {
        return this.negate().plus(mathContext);
    }
    
    public BigDecimal plus() {
        return this;
    }
    
    public BigDecimal plus(MathContext mathContext) {
        if (mathContext.precision == 0) {
            return this;
        }
        return doRound(this, mathContext);
    }
    
    public int signum() {
        return (this.intCompact != Long.MIN_VALUE) ? Long.signum(this.intCompact) : this.intVal.signum();
    }
    
    public int scale() {
        return this.scale;
    }
    
    public int precision() {
        int precision = this.precision;
        if (precision == 0) {
            long intCompact = this.intCompact;
            if (intCompact != Long.MIN_VALUE) {
                precision = longDigitLength(intCompact);
            }
            else {
                precision = bigDigitLength(this.intVal);
            }
            this.precision = precision;
        }
        return precision;
    }
    
    public BigInteger unscaledValue() {
        return this.inflated();
    }
    
    public BigDecimal round(MathContext mathContext) {
        return this.plus(mathContext);
    }
    
    public BigDecimal setScale(int n, RoundingMode roundingMode) {
        return this.setScale(n, roundingMode.oldMode);
    }
    
    public BigDecimal setScale(int n, int n2) {
        if (n2 < 0 || n2 > 7) {
            throw new IllegalArgumentException("Invalid rounding mode");
        }
        int scale = this.scale;
        if (n == scale) {
            return this;
        }
        if (this.signum() == 0) {
            return zeroValueOf(n);
        }
        if (this.intCompact != Long.MIN_VALUE) {
            long intCompact = this.intCompact;
            if (n > scale) {
                int checkScale4 = this.checkScale((long)n - (long)scale);
                long longMultiplyPowerTen;
                if ((longMultiplyPowerTen = longMultiplyPowerTen(intCompact, checkScale4)) != Long.MIN_VALUE) {
                    return valueOf(longMultiplyPowerTen, n);
                }
                return new BigDecimal(this.bigMultiplyPowerTen(checkScale4), Long.MIN_VALUE, n, (this.precision > 0) ? (this.precision + checkScale4) : 0);
            }
            else {
                int checkScale3 = this.checkScale((long)scale - (long)n);
                if (checkScale3 < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
                    return divideAndRound(intCompact, BigDecimal.LONG_TEN_POWERS_TABLE[checkScale3], n, n2, n);
                }
                return divideAndRound(this.inflated(), bigTenToThe(checkScale3), n, n2, n);
            }
        }
        else {
            if (n > scale) {
                int checkScale2 = this.checkScale((long)n - (long)scale);
                return new BigDecimal(bigMultiplyPowerTen(this.intVal, checkScale2), Long.MIN_VALUE, n, (this.precision > 0) ? (this.precision + checkScale2) : 0);
            }
            int checkScale = this.checkScale((long)scale - (long)n);
            if (checkScale < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
                return divideAndRound(this.intVal, BigDecimal.LONG_TEN_POWERS_TABLE[checkScale], n, n2, n);
            }
            return divideAndRound(this.intVal, bigTenToThe(checkScale), n, n2, n);
        }
    }
    
    public BigDecimal setScale(int n) {
        return this.setScale(n, 7);
    }
    
    public BigDecimal movePointLeft(int n) {
        BigDecimal bigDecimal = new BigDecimal(this.intVal, this.intCompact, this.checkScale((long)this.scale + (long)n), 0);
        return (bigDecimal.scale < 0) ? bigDecimal.setScale(0, 7) : bigDecimal;
    }
    
    public BigDecimal movePointRight(int n) {
        BigDecimal bigDecimal = new BigDecimal(this.intVal, this.intCompact, this.checkScale((long)this.scale - (long)n), 0);
        return (bigDecimal.scale < 0) ? bigDecimal.setScale(0, 7) : bigDecimal;
    }
    
    public BigDecimal scaleByPowerOfTen(int n) {
        return new BigDecimal(this.intVal, this.intCompact, this.checkScale((long)this.scale - (long)n), this.precision);
    }
    
    public BigDecimal stripTrailingZeros() {
        if (this.intCompact == 0L || (this.intVal != null && this.intVal.signum() == 0)) {
            return BigDecimal.ZERO;
        }
        if (this.intCompact != Long.MIN_VALUE) {
            return createAndStripZerosToMatchScale(this.intCompact, this.scale, Long.MIN_VALUE);
        }
        return createAndStripZerosToMatchScale(this.intVal, this.scale, Long.MIN_VALUE);
    }
    
    @Override
    public int compareTo(BigDecimal bigDecimal) {
        if (this.scale == bigDecimal.scale) {
            long intCompact = this.intCompact;
            long intCompact2 = bigDecimal.intCompact;
            if (intCompact != Long.MIN_VALUE && intCompact2 != Long.MIN_VALUE) {
                return (intCompact != intCompact2) ? ((intCompact > intCompact2) ? 1 : -1) : 0;
            }
        }
        int signum = this.signum();
        int signum2 = bigDecimal.signum();
        if (signum != signum2) {
            return (signum > signum2) ? 1 : -1;
        }
        if (signum == 0) {
            return 0;
        }
        int compareMagnitude = this.compareMagnitude(bigDecimal);
        return (signum > 0) ? compareMagnitude : (-compareMagnitude);
    }
    
    private int compareMagnitude(BigDecimal bigDecimal) {
        long n = bigDecimal.intCompact;
        long n2 = this.intCompact;
        if (n2 == 0L) {
            return (n == 0L) ? 0 : -1;
        }
        if (n == 0L) {
            return 1;
        }
        long n3 = (long)this.scale - (long)bigDecimal.scale;
        if (n3 != 0L) {
            long n4 = (long)this.precision() - (long)this.scale;
            long n5 = (long)bigDecimal.precision() - (long)bigDecimal.scale;
            if (n4 < n5) {
                return -1;
            }
            if (n4 > n5) {
                return 1;
            }
            if (n3 < 0L) {
                if (n3 > -2147483648L && (n2 == Long.MIN_VALUE || (n2 = longMultiplyPowerTen(n2, (int)(-n3))) == Long.MIN_VALUE) && n == Long.MIN_VALUE) {
                    return this.bigMultiplyPowerTen((int)(-n3)).compareMagnitude(bigDecimal.intVal);
                }
            }
            else if (n3 <= 2147483647L && (n == Long.MIN_VALUE || (n = longMultiplyPowerTen(n, (int)n3)) == Long.MIN_VALUE) && n2 == Long.MIN_VALUE) {
                return this.intVal.compareMagnitude(bigDecimal.bigMultiplyPowerTen((int)n3));
            }
        }
        if (n2 != Long.MIN_VALUE) {
            return (n != Long.MIN_VALUE) ? longCompareMagnitude(n2, n) : -1;
        }
        if (n != Long.MIN_VALUE) {
            return 1;
        }
        return this.intVal.compareMagnitude(bigDecimal.intVal);
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof BigDecimal)) {
            return false;
        }
        BigDecimal bigDecimal = (BigDecimal)o;
        if (o == this) {
            return true;
        }
        if (this.scale != bigDecimal.scale) {
            return false;
        }
        long intCompact = this.intCompact;
        long n = bigDecimal.intCompact;
        if (intCompact != Long.MIN_VALUE) {
            if (n == Long.MIN_VALUE) {
                n = compactValFor(bigDecimal.intVal);
            }
            return n == intCompact;
        }
        else {
            if (n != Long.MIN_VALUE) {
                return n == compactValFor(this.intVal);
            }
            return this.inflated().equals(bigDecimal.inflated());
        }
    }
    
    public BigDecimal min(BigDecimal bigDecimal) {
        return (this.compareTo(bigDecimal) <= 0) ? this : bigDecimal;
    }
    
    public BigDecimal max(BigDecimal bigDecimal) {
        return (this.compareTo(bigDecimal) >= 0) ? this : bigDecimal;
    }
    
    @Override
    public int hashCode() {
        if (this.intCompact != Long.MIN_VALUE) {
            long n = (this.intCompact < 0L) ? (-this.intCompact) : this.intCompact;
            int n2 = (int)((long)((int)(n >>> 32) * 31) + (n & 0xFFFFFFFFL));
            return 31 * ((this.intCompact < 0L) ? (-n2) : n2) + this.scale;
        }
        return 31 * this.intVal.hashCode() + this.scale;
    }
    
    @Override
    public String toString() {
        String stringCache = this.stringCache;
        if (stringCache == null) {
            stringCache = (this.stringCache = this.layoutChars(true));
            return stringCache;
        }
        return stringCache;
    }
    
    public String toEngineeringString() {
        return this.layoutChars(false);
    }
    
    public String toPlainString() {
        if (this.scale == 0) {
            if (this.intCompact != Long.MIN_VALUE) {
                return Long.toString(this.intCompact);
            }
            return this.intVal.toString();
        }
        else if (this.scale < 0) {
            if (this.signum() == 0) {
                return "0";
            }
            int checkScaleNonZero = checkScaleNonZero(-(long)this.scale);
            StringBuilder sb;
            if (this.intCompact != Long.MIN_VALUE) {
                sb = new StringBuilder(20 + checkScaleNonZero);
                sb.append(this.intCompact);
            }
            else {
                String string = this.intVal.toString();
                sb = new StringBuilder(string.length() + checkScaleNonZero);
                sb.append(string);
            }
            for (int i = 0; i < checkScaleNonZero; ++i) {
                sb.append('0');
            }
            return sb.toString();
        }
        else {
            String s;
            if (this.intCompact != Long.MIN_VALUE) {
                s = Long.toString(Math.abs(this.intCompact));
            }
            else {
                s = this.intVal.abs().toString();
            }
            return this.getValueString(this.signum(), s, this.scale);
        }
    }
    
    private String getValueString(int n, String str, int n2) {
        int offset = str.length() - n2;
        if (offset == 0) {
            return ((n < 0) ? "-0." : "0.") + str;
        }
        StringBuilder sb;
        if (offset > 0) {
            sb = new StringBuilder(str);
            sb.insert(offset, '.');
            if (n < 0) {
                sb.insert(0, '-');
            }
        }
        else {
            sb = new StringBuilder(3 - offset + str.length());
            sb.append((n < 0) ? "-0." : "0.");
            for (int i = 0; i < -offset; ++i) {
                sb.append('0');
            }
            sb.append(str);
        }
        return sb.toString();
    }
    
    public BigInteger toBigInteger() {
        return this.setScale(0, 1).inflated();
    }
    
    public BigInteger toBigIntegerExact() {
        return this.setScale(0, 7).inflated();
    }
    
    @Override
    public long longValue() {
        if (this.intCompact != Long.MIN_VALUE && this.scale == 0) {
            return this.intCompact;
        }
        if (this.signum() == 0 || this.fractionOnly() || this.scale <= -64) {
            return 0L;
        }
        return this.toBigInteger().longValue();
    }
    
    private boolean fractionOnly() {
        assert this.signum() != 0;
        return this.precision() - this.scale <= 0;
    }
    
    public long longValueExact() {
        if (this.intCompact != Long.MIN_VALUE && this.scale == 0) {
            return this.intCompact;
        }
        if (this.signum() == 0) {
            return 0L;
        }
        if (this.fractionOnly()) {
            throw new ArithmeticException("Rounding necessary");
        }
        if (this.precision() - this.scale > 19) {
            throw new ArithmeticException("Overflow");
        }
        BigDecimal setScale = this.setScale(0, 7);
        if (setScale.precision() >= 19) {
            BigDecimal.LongOverflow.check(setScale);
        }
        return setScale.inflated().longValue();
    }
    
    @Override
    public int intValue() {
        return (this.intCompact != Long.MIN_VALUE && this.scale == 0) ? ((int)this.intCompact) : ((int)this.longValue());
    }
    
    public int intValueExact() {
        long longValueExact = this.longValueExact();
        if ((long)(int)longValueExact != longValueExact) {
            throw new ArithmeticException("Overflow");
        }
        return (int)longValueExact;
    }
    
    public short shortValueExact() {
        long longValueExact = this.longValueExact();
        if ((long)(short)(int)longValueExact != longValueExact) {
            throw new ArithmeticException("Overflow");
        }
        return (short)(int)longValueExact;
    }
    
    public byte byteValueExact() {
        long longValueExact = this.longValueExact();
        if ((long)(byte)(int)longValueExact != longValueExact) {
            throw new ArithmeticException("Overflow");
        }
        return (byte)(int)longValueExact;
    }
    
    @Override
    public float floatValue() {
        if (this.intCompact != Long.MIN_VALUE) {
            if (this.scale == 0) {
                return (float)this.intCompact;
            }
            if (Math.abs(this.intCompact) < 4194304L) {
                if (this.scale > 0 && this.scale < BigDecimal.float10pow.length) {
                    return (float)this.intCompact / BigDecimal.float10pow[this.scale];
                }
                if (this.scale < 0 && this.scale > -BigDecimal.float10pow.length) {
                    return (float)this.intCompact * BigDecimal.float10pow[-this.scale];
                }
            }
        }
        return Float.parseFloat(this.toString());
    }
    
    @Override
    public double doubleValue() {
        if (this.intCompact != Long.MIN_VALUE) {
            if (this.scale == 0) {
                return (double)this.intCompact;
            }
            if (Math.abs(this.intCompact) < 4503599627370496L) {
                if (this.scale > 0 && this.scale < BigDecimal.double10pow.length) {
                    return (double)this.intCompact / BigDecimal.double10pow[this.scale];
                }
                if (this.scale < 0 && this.scale > -BigDecimal.double10pow.length) {
                    return (double)this.intCompact * BigDecimal.double10pow[-this.scale];
                }
            }
        }
        return Double.parseDouble(this.toString());
    }
    
    public BigDecimal ulp() {
        return valueOf(1L, this.scale(), 1);
    }
    
    private String layoutChars(boolean b) {
        if (this.scale == 0) {
            return (this.intCompact != Long.MIN_VALUE) ? Long.toString(this.intCompact) : this.intVal.toString();
        }
        if (this.scale == 2 && this.intCompact >= 0L && this.intCompact < 2147483647L) {
            int n3 = (int)this.intCompact % 100;
            return Integer.toString((int)this.intCompact / 100) + 46 + BigDecimal.StringBuilderHelper.DIGIT_TENS[n3] + BigDecimal.StringBuilderHelper.DIGIT_ONES[n3];
        }
        BigDecimal.StringBuilderHelper stringBuilderHelper = (BigDecimal.StringBuilderHelper)BigDecimal.threadLocalStringBuilderHelper.get();
        int putIntCompact;
        char[] str;
        if (this.intCompact != Long.MIN_VALUE) {
            putIntCompact = stringBuilderHelper.putIntCompact(Math.abs(this.intCompact));
            str = stringBuilderHelper.getCompactCharArray();
        }
        else {
            putIntCompact = 0;
            str = this.intVal.abs().toString().toCharArray();
        }
        StringBuilder stringBuilder = stringBuilderHelper.getStringBuilder();
        if (this.signum() < 0) {
            stringBuilder.append('-');
        }
        int n = str.length - putIntCompact;
        long lng = -(long)this.scale + (long)(n - 1);
        if (this.scale >= 0 && lng >= -6L) {
            int j = this.scale - n;
            if (j >= 0) {
                stringBuilder.append('0');
                stringBuilder.append('.');
                while (j > 0) {
                    stringBuilder.append('0');
                    --j;
                }
                stringBuilder.append(str, putIntCompact, n);
            }
            else {
                stringBuilder.append(str, putIntCompact, -j);
                stringBuilder.append('.');
                stringBuilder.append(str, -j + putIntCompact, this.scale);
            }
        }
        else {
            if (b) {
                stringBuilder.append(str[putIntCompact]);
                if (n > 1) {
                    stringBuilder.append('.');
                    stringBuilder.append(str, putIntCompact + 1, n - 1);
                }
            }
            else {
                int n2 = (int)(lng % 3L);
                if (n2 < 0) {
                    n2 += 3;
                }
                lng -= (long)n2;
                ++n2;
                if (this.signum() == 0) {
                    switch (n2) {
                        case 1: {
                            stringBuilder.append('0');
                            break;
                        }
                        case 2: {
                            stringBuilder.append("0.00");
                            lng += 3L;
                            break;
                        }
                        case 3: {
                            stringBuilder.append("0.0");
                            lng += 3L;
                            break;
                        }
                        default: {
                            throw new AssertionError("Unexpected sig value " + n2);
                        }
                    }
                }
                else if (n2 >= n) {
                    stringBuilder.append(str, putIntCompact, n);
                    for (int i = n2 - n; i > 0; --i) {
                        stringBuilder.append('0');
                    }
                }
                else {
                    stringBuilder.append(str, putIntCompact, n2);
                    stringBuilder.append('.');
                    stringBuilder.append(str, putIntCompact + n2, n - n2);
                }
            }
            if (lng != 0L) {
                stringBuilder.append('E');
                if (lng > 0L) {
                    stringBuilder.append('+');
                }
                stringBuilder.append(lng);
            }
        }
        return stringBuilder.toString();
    }
    
    private static BigInteger bigTenToThe(int n) {
        if (n < 0) {
            return BigInteger.ZERO;
        }
        if (n < BigDecimal.BIG_TEN_POWERS_TABLE_MAX) {
            BigInteger[] big_TEN_POWERS_TABLE = BigDecimal.BIG_TEN_POWERS_TABLE;
            if (n < big_TEN_POWERS_TABLE.length) {
                return big_TEN_POWERS_TABLE[n];
            }
            return expandBigIntegerTenPowers(n);
        }
        else {
            return BigInteger.TEN.pow(n);
        }
    }
    
    private static BigInteger expandBigIntegerTenPowers(int n) {
        synchronized (BigDecimal.class) {
            BigInteger[] big_TEN_POWERS_TABLE = BigDecimal.BIG_TEN_POWERS_TABLE;
            int i = big_TEN_POWERS_TABLE.length;
            if (i <= n) {
                int j;
                for (j = i << 1; j <= n; j <<= 1) {}
                big_TEN_POWERS_TABLE = (BigInteger[])Arrays.copyOf(big_TEN_POWERS_TABLE, j);
                for (; i < j; ++i) {
                    big_TEN_POWERS_TABLE[i] = big_TEN_POWERS_TABLE[i - 1].multiply(BigInteger.TEN);
                }
                BigDecimal.BIG_TEN_POWERS_TABLE = big_TEN_POWERS_TABLE;
            }
            return big_TEN_POWERS_TABLE[n];
        }
    }
    
    private static long longMultiplyPowerTen(long a, int n) {
        if (a == 0L || n <= 0) {
            return a;
        }
        long[] long_TEN_POWERS_TABLE = BigDecimal.LONG_TEN_POWERS_TABLE;
        long[] thresholds_TABLE = BigDecimal.THRESHOLDS_TABLE;
        if (n < long_TEN_POWERS_TABLE.length && n < thresholds_TABLE.length) {
            long n2 = long_TEN_POWERS_TABLE[n];
            if (a == 1L) {
                return n2;
            }
            if (Math.abs(a) <= thresholds_TABLE[n]) {
                return a * n2;
            }
        }
        return Long.MIN_VALUE;
    }
    
    private BigInteger bigMultiplyPowerTen(int n) {
        if (n <= 0) {
            return this.inflated();
        }
        if (this.intCompact != Long.MIN_VALUE) {
            return bigTenToThe(n).multiply(this.intCompact);
        }
        return this.intVal.multiply(bigTenToThe(n));
    }
    
    private BigInteger inflated() {
        if (this.intVal == null) {
            return BigInteger.valueOf(this.intCompact);
        }
        return this.intVal;
    }
    
    private static void matchScale(BigDecimal[] array) {
        if (array[0].scale == array[1].scale) {
            return;
        }
        if (array[0].scale < array[1].scale) {
            array[0] = array[0].setScale(array[1].scale, 7);
        }
        else if (array[1].scale < array[0].scale) {
            array[1] = array[1].setScale(array[0].scale, 7);
        }
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.intVal == null) {
            throw new StreamCorruptedException("BigDecimal: null intVal in stream");
        }
        BigDecimal.UnsafeHolder.setIntCompactVolatile(this, compactValFor(this.intVal));
        return;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.intVal == null) {
            BigDecimal.UnsafeHolder.setIntValVolatile(this, BigInteger.valueOf(this.intCompact));
        }
        objectOutputStream.defaultWriteObject();
    }
    
    static int longDigitLength(long i) {
        assert i != Long.MIN_VALUE;
        if (i < 0L) {
            i = -i;
        }
        if (i < 10L) {
            return 1;
        }
        int n = (64 - Long.numberOfLeadingZeros(i) + 1) * 1233 >>> 12;
        long[] long_TEN_POWERS_TABLE = BigDecimal.LONG_TEN_POWERS_TABLE;
        return (n >= long_TEN_POWERS_TABLE.length || i < long_TEN_POWERS_TABLE[n]) ? n : (n + 1);
    }
    
    private static int bigDigitLength(BigInteger bigInteger) {
        if (bigInteger.signum == 0) {
            return 1;
        }
        int n = (int)(((long)bigInteger.bitLength() + 1L) * 646456993L >>> 31);
        return (bigInteger.compareMagnitude(bigTenToThe(n)) < 0) ? n : (n + 1);
    }
    
    private int checkScale(long n) {
        int n2 = (int)n;
        if ((long)n2 != n) {
            n2 = ((n > 2147483647L) ? Integer.MAX_VALUE : Integer.MIN_VALUE);
            BigInteger intVal;
            if (this.intCompact != 0L && ((intVal = this.intVal) == null || intVal.signum() != 0)) {
                throw new ArithmeticException((n2 > 0) ? "Underflow" : "Overflow");
            }
        }
        return n2;
    }
    
    private static long compactValFor(BigInteger bigInteger) {
        int[] mag = bigInteger.mag;
        int length = mag.length;
        if (length == 0) {
            return 0L;
        }
        int n = mag[0];
        if (length > 2 || (length == 2 && n < 0)) {
            return Long.MIN_VALUE;
        }
        long n2 = (length == 2) ? (((long)mag[1] & 0xFFFFFFFFL) + ((long)n << 32)) : ((long)n & 0xFFFFFFFFL);
        return (bigInteger.signum < 0) ? (-n2) : n2;
    }
    
    private static int longCompareMagnitude(long n, long n2) {
        if (n < 0L) {
            n = -n;
        }
        if (n2 < 0L) {
            n2 = -n2;
        }
        return (n < n2) ? -1 : ((n == n2) ? 0 : 1);
    }
    
    private static int saturateLong(long n) {
        int n2 = (int)n;
        return (n == (long)n2) ? n2 : ((n < 0L) ? Integer.MIN_VALUE : Integer.MAX_VALUE);
    }
    
    private static void print(String s, BigDecimal bigDecimal) {
        System.err.format("%s:\tintCompact %d\tintVal %d\tscale %d\tprecision %d%n", new Object[] { s, Long.valueOf(bigDecimal.intCompact), bigDecimal.intVal, Integer.valueOf(bigDecimal.scale), Integer.valueOf(bigDecimal.precision) });
    }
    
    private BigDecimal audit() {
        if (this.intCompact == Long.MIN_VALUE) {
            if (this.intVal == null) {
                print("audit", this);
                throw new AssertionError("null intVal");
            }
            if (this.precision > 0 && this.precision != bigDigitLength(this.intVal)) {
                print("audit", this);
                throw new AssertionError("precision mismatch");
            }
        }
        else {
            if (this.intVal != null) {
                long longValue = this.intVal.longValue();
                if (longValue != this.intCompact) {
                    print("audit", this);
                    throw new AssertionError("Inconsistent state, intCompact=" + this.intCompact + "\t intVal=" + longValue);
                }
            }
            if (this.precision > 0 && this.precision != longDigitLength(this.intCompact)) {
                print("audit", this);
                throw new AssertionError("precision mismatch");
            }
        }
        return this;
    }
    
    private static int checkScaleNonZero(long n) {
        int n2 = (int)n;
        if ((long)n2 != n) {
            throw new ArithmeticException((n2 > 0) ? "Underflow" : "Overflow");
        }
        return n2;
    }
    
    private static int checkScale(long n, long n2) {
        int n3 = (int)n2;
        if ((long)n3 != n2) {
            n3 = ((n2 > 2147483647L) ? Integer.MAX_VALUE : Integer.MIN_VALUE);
            if (n != 0L) {
                throw new ArithmeticException((n3 > 0) ? "Underflow" : "Overflow");
            }
        }
        return n3;
    }
    
    private static int checkScale(BigInteger bigInteger, long n) {
        int n2 = (int)n;
        if ((long)n2 != n) {
            n2 = ((n > 2147483647L) ? Integer.MAX_VALUE : Integer.MIN_VALUE);
            if (bigInteger.signum() != 0) {
                throw new ArithmeticException((n2 > 0) ? "Underflow" : "Overflow");
            }
        }
        return n2;
    }
    
    private static BigDecimal doRound(BigDecimal bigDecimal, MathContext mathContext) {
        int precision = mathContext.precision;
        boolean n = false;
        if (precision > 0) {
            BigInteger bigInteger = bigDecimal.intVal;
            long n2 = bigDecimal.intCompact;
            int n3 = bigDecimal.scale;
            int n4 = bigDecimal.precision();
            int oldMode = mathContext.roundingMode.oldMode;
            if (n2 == Long.MIN_VALUE) {
                int i = n4 - precision;
                while (i > 0) {
                    n3 = checkScaleNonZero((long)n3 - (long)i);
                    bigInteger = divideAndRoundByTenPow(bigInteger, i, oldMode);
                    n = true;
                    n2 = compactValFor(bigInteger);
                    if (n2 != Long.MIN_VALUE) {
                        n4 = longDigitLength(n2);
                        break;
                    }
                    n4 = bigDigitLength(bigInteger);
                    i = n4 - precision;
                }
            }
            if (n2 != Long.MIN_VALUE) {
                for (int j = n4 - precision; j > 0; j = n4 - precision, bigInteger = null) {
                    n3 = checkScaleNonZero((long)n3 - (long)j);
                    n2 = divideAndRound(n2, BigDecimal.LONG_TEN_POWERS_TABLE[j], mathContext.roundingMode.oldMode);
                    n = true;
                    n4 = longDigitLength(n2);
                }
            }
            return n ? new BigDecimal(bigInteger, n2, n3, n4) : bigDecimal;
        }
        else {
            return bigDecimal;
        }
    }
    
    private static BigDecimal doRound(long divideAndRound, int checkScaleNonZero, MathContext mathContext) {
        int precision = mathContext.precision;
        if (precision > 0 && precision < 19) {
            int n = longDigitLength(divideAndRound);
            for (int i = n - precision; i > 0; i = n - precision) {
                checkScaleNonZero = checkScaleNonZero((long)checkScaleNonZero - (long)i);
                divideAndRound = divideAndRound(divideAndRound, BigDecimal.LONG_TEN_POWERS_TABLE[i], mathContext.roundingMode.oldMode);
                n = longDigitLength(divideAndRound);
            }
            return valueOf(divideAndRound, checkScaleNonZero, n);
        }
        return valueOf(divideAndRound, checkScaleNonZero);
    }
    
    private static BigDecimal doRound(BigInteger divideAndRoundByTenPow, int n, MathContext mathContext) {
        int precision = mathContext.precision;
        int n2 = 0;
        if (precision > 0) {
            long n3 = compactValFor(divideAndRoundByTenPow);
            int oldMode = mathContext.roundingMode.oldMode;
            if (n3 == Long.MIN_VALUE) {
                n2 = bigDigitLength(divideAndRoundByTenPow);
                int i = n2 - precision;
                while (i > 0) {
                    n = checkScaleNonZero((long)n - (long)i);
                    divideAndRoundByTenPow = divideAndRoundByTenPow(divideAndRoundByTenPow, i, oldMode);
                    n3 = compactValFor(divideAndRoundByTenPow);
                    if (n3 != Long.MIN_VALUE) {
                        break;
                    }
                    n2 = bigDigitLength(divideAndRoundByTenPow);
                    i = n2 - precision;
                }
            }
            if (n3 != Long.MIN_VALUE) {
                int n4 = longDigitLength(n3);
                for (int j = n4 - precision; j > 0; j = n4 - precision) {
                    n = checkScaleNonZero((long)n - (long)j);
                    n3 = divideAndRound(n3, BigDecimal.LONG_TEN_POWERS_TABLE[j], mathContext.roundingMode.oldMode);
                    n4 = longDigitLength(n3);
                }
                return valueOf(n3, n, n4);
            }
        }
        return new BigDecimal(divideAndRoundByTenPow, Long.MIN_VALUE, n, n2);
    }
    
    private static BigInteger divideAndRoundByTenPow(BigInteger bigInteger, int n, int n2) {
        if (n < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
            bigInteger = divideAndRound(bigInteger, BigDecimal.LONG_TEN_POWERS_TABLE[n], n2);
            return bigInteger;
        }
        bigInteger = divideAndRound(bigInteger, bigTenToThe(n), n2);
        return bigInteger;
    }
    
    private static BigDecimal divideAndRound(long n, long n2, int n3, int n4, int n5) {
        long n6 = n / n2;
        if (n4 == 1 && n3 == n5) {
            return valueOf(n6, n3);
        }
        long n7 = n % n2;
        int n8 = (n < 0L == n2 < 0L) ? 1 : -1;
        if (n7 != 0L) {
            return valueOf(needIncrement(n2, n4, n8, n6, n7) ? (n6 + (long)n8) : n6, n3);
        }
        if (n5 != n3) {
            return createAndStripZerosToMatchScale(n6, n3, (long)n5);
        }
        return valueOf(n6, n3);
    }
    
    private static long divideAndRound(long n, long n2, int n3) {
        long n4 = n / n2;
        if (n3 == 1) {
            return n4;
        }
        long n5 = n % n2;
        int n6 = (n < 0L == n2 < 0L) ? 1 : -1;
        if (n5 != 0L) {
            return needIncrement(n2, n3, n6, n4, n5) ? (n4 + (long)n6) : n4;
        }
        return n4;
    }
    
    private static boolean commonNeedIncrement(int i, int n, int n2, boolean b) {
        switch (i) {
            case 7: {
                throw new ArithmeticException("Rounding necessary");
            }
            case 0: {
                return true;
            }
            case 1: {
                return false;
            }
            case 2: {
                return n > 0;
            }
            case 3: {
                return n < 0;
            }
            default: {
                assert i >= 4 && i <= 6 : "Unexpected rounding mode" + RoundingMode.valueOf(i);
                if (n2 < 0) {
                    return false;
                }
                if (n2 > 0) {
                    return true;
                }
                assert n2 == 0;
                switch (i) {
                    case 5: {
                        return false;
                    }
                    case 4: {
                        return true;
                    }
                    case 6: {
                        return b;
                    }
                    default: {
                        throw new AssertionError("Unexpected rounding mode" + i);
                    }
                }
            }
        }
    }
    
    private static boolean needIncrement(long n, int n2, int n3, long n4, long n5) {
        assert n5 != 0L;
        int longCompareMagnitude;
        if (n5 <= -4611686018427387904L || n5 > 4611686018427387903L) {
            longCompareMagnitude = 1;
        }
        else {
            longCompareMagnitude = longCompareMagnitude(2L * n5, n);
        }
        return commonNeedIncrement(n2, n3, longCompareMagnitude, (n4 & 0x1L) != 0x0L);
    }
    
    private static BigInteger divideAndRound(BigInteger bigInteger, long n, int n2) {
        MutableBigInteger mutableBigInteger = new MutableBigInteger(bigInteger.mag);
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        long divide = mutableBigInteger.divide(n, mutableBigInteger2);
        int n3 = divide == 0L;
        int n4 = (n < 0L) ? (-bigInteger.signum) : bigInteger.signum;
        if (n3 == 0 && needIncrement(n, n2, n4, mutableBigInteger2, divide)) {
            mutableBigInteger2.add(MutableBigInteger.ONE);
        }
        return mutableBigInteger2.toBigInteger(n4);
    }
    
    private static BigDecimal divideAndRound(BigInteger bigInteger, long n, int n2, int n3, int n4) {
        MutableBigInteger mutableBigInteger = new MutableBigInteger(bigInteger.mag);
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        long divide = mutableBigInteger.divide(n, mutableBigInteger2);
        int n5 = divide == 0L;
        int n6 = (n < 0L) ? (-bigInteger.signum) : bigInteger.signum;
        if (n5 == 0) {
            if (needIncrement(n, n3, n6, mutableBigInteger2, divide)) {
                mutableBigInteger2.add(MutableBigInteger.ONE);
            }
            return mutableBigInteger2.toBigDecimal(n6, n2);
        }
        else if (n4 != n2) {
            long compactValue = mutableBigInteger2.toCompactValue(n6);
            if (compactValue != Long.MIN_VALUE) {
                return createAndStripZerosToMatchScale(compactValue, n2, (long)n4);
            }
            return createAndStripZerosToMatchScale(mutableBigInteger2.toBigInteger(n6), n2, (long)n4);
        }
        else {
            return mutableBigInteger2.toBigDecimal(n6, n2);
        }
    }
    
    private static boolean needIncrement(long n, int n2, int n3, MutableBigInteger mutableBigInteger, long n4) {
        assert n4 != 0L;
        int longCompareMagnitude;
        if (n4 <= -4611686018427387904L || n4 > 4611686018427387903L) {
            longCompareMagnitude = 1;
        }
        else {
            longCompareMagnitude = longCompareMagnitude(2L * n4, n);
        }
        return commonNeedIncrement(n2, n3, longCompareMagnitude, mutableBigInteger.isOdd());
    }
    
    private static BigInteger divideAndRound(BigInteger bigInteger, BigInteger bigInteger2, int n) {
        MutableBigInteger mutableBigInteger = new MutableBigInteger(bigInteger.mag);
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(bigInteger2.mag);
        MutableBigInteger divide = mutableBigInteger.divide(mutableBigInteger3, mutableBigInteger2);
        int zero = divide.isZero();
        int n2 = (bigInteger.signum != bigInteger2.signum) ? -1 : 1;
        if (zero == 0 && needIncrement(mutableBigInteger3, n, n2, mutableBigInteger2, divide)) {
            mutableBigInteger2.add(MutableBigInteger.ONE);
        }
        return mutableBigInteger2.toBigInteger(n2);
    }
    
    private static BigDecimal divideAndRound(BigInteger bigInteger, BigInteger bigInteger2, int n, int n2, int n3) {
        MutableBigInteger mutableBigInteger = new MutableBigInteger(bigInteger.mag);
        MutableBigInteger mutableBigInteger2 = new MutableBigInteger();
        MutableBigInteger mutableBigInteger3 = new MutableBigInteger(bigInteger2.mag);
        MutableBigInteger divide = mutableBigInteger.divide(mutableBigInteger3, mutableBigInteger2);
        int zero = divide.isZero();
        int n4 = (bigInteger.signum != bigInteger2.signum) ? -1 : 1;
        if (zero == 0) {
            if (needIncrement(mutableBigInteger3, n2, n4, mutableBigInteger2, divide)) {
                mutableBigInteger2.add(MutableBigInteger.ONE);
            }
            return mutableBigInteger2.toBigDecimal(n4, n);
        }
        else if (n3 != n) {
            long compactValue = mutableBigInteger2.toCompactValue(n4);
            if (compactValue != Long.MIN_VALUE) {
                return createAndStripZerosToMatchScale(compactValue, n, (long)n3);
            }
            return createAndStripZerosToMatchScale(mutableBigInteger2.toBigInteger(n4), n, (long)n3);
        }
        else {
            return mutableBigInteger2.toBigDecimal(n4, n);
        }
    }
    
    private static boolean needIncrement(MutableBigInteger mutableBigInteger, int n, int n2, MutableBigInteger mutableBigInteger2, MutableBigInteger mutableBigInteger3) {
        assert !mutableBigInteger3.isZero();
        return commonNeedIncrement(n, n2, mutableBigInteger3.compareHalf(mutableBigInteger), mutableBigInteger2.isOdd());
    }
    
    private static BigDecimal createAndStripZerosToMatchScale(BigInteger bigInteger, int checkScale, long n) {
        while (bigInteger.compareMagnitude(BigInteger.TEN) >= 0 && (long)checkScale > n) {
            if (bigInteger.testBit(0)) {
                break;
            }
            BigInteger[] divideAndRemainder = bigInteger.divideAndRemainder(BigInteger.TEN);
            if (divideAndRemainder[1].signum() != 0) {
                break;
            }
            bigInteger = divideAndRemainder[0];
            checkScale = checkScale(bigInteger, (long)checkScale - 1L);
        }
        return valueOf(bigInteger, checkScale, 0);
    }
    
    private static BigDecimal createAndStripZerosToMatchScale(long a, int checkScale, long n) {
        while (Math.abs(a) >= 10L && (long)checkScale > n) {
            if ((a & 0x1L) != 0x0L) {
                break;
            }
            if (a % 10L != 0L) {
                break;
            }
            a /= 10L;
            checkScale = checkScale(a, (long)checkScale - 1L);
        }
        return valueOf(a, checkScale);
    }
    
    private static BigDecimal stripZerosToMatchScale(BigInteger bigInteger, long n, int n2, int n3) {
        if (n != Long.MIN_VALUE) {
            return createAndStripZerosToMatchScale(n, n2, (long)n3);
        }
        return createAndStripZerosToMatchScale((bigInteger == null) ? BigDecimal.INFLATED_BIGINT : bigInteger, n2, (long)n3);
    }
    
    private static long add(long n, long n2) {
        long n3 = n + n2;
        if (((n3 ^ n) & (n3 ^ n2)) >= 0L) {
            return n3;
        }
        return Long.MIN_VALUE;
    }
    
    private static BigDecimal add(long n, long n2, int n3) {
        long add = add(n, n2);
        if (add != Long.MIN_VALUE) {
            return valueOf(add, n3);
        }
        return new BigDecimal(BigInteger.valueOf(n).add(n2), n3);
    }
    
    private static BigDecimal add(long n, int n2, long n3, int n4) {
        long n5 = (long)n2 - (long)n4;
        if (n5 == 0L) {
            return add(n, n3, n2);
        }
        if (n5 < 0L) {
            int checkScale2 = checkScale(n, -n5);
            long longMultiplyPowerTen2 = longMultiplyPowerTen(n, checkScale2);
            if (longMultiplyPowerTen2 != Long.MIN_VALUE) {
                return add(longMultiplyPowerTen2, n3, n4);
            }
            BigInteger add2 = bigMultiplyPowerTen(n, checkScale2).add(n3);
            return ((n ^ n3) >= 0L) ? new BigDecimal(add2, Long.MIN_VALUE, n4, 0) : valueOf(add2, n4, 0);
        }
        else {
            int checkScale = checkScale(n3, n5);
            long longMultiplyPowerTen = longMultiplyPowerTen(n3, checkScale);
            if (longMultiplyPowerTen != Long.MIN_VALUE) {
                return add(n, longMultiplyPowerTen, n2);
            }
            BigInteger add = bigMultiplyPowerTen(n3, checkScale).add(n);
            return ((n ^ n3) >= 0L) ? new BigDecimal(add, Long.MIN_VALUE, n2, 0) : valueOf(add, n2, 0);
        }
    }
    
    private static BigDecimal add(long i, int n, BigInteger bigMultiplyPowerTen, int n2) {
        long n3 = (long)n - (long)n2;
        int n4 = Long.signum(i) == bigMultiplyPowerTen.signum;
        BigInteger bigInteger;
        if (n3 < 0L) {
            int checkScale = checkScale(i, -n3);
            n = n2;
            long longMultiplyPowerTen = longMultiplyPowerTen(i, checkScale);
            if (longMultiplyPowerTen == Long.MIN_VALUE) {
                bigInteger = bigMultiplyPowerTen.add(bigMultiplyPowerTen(i, checkScale));
            }
            else {
                bigInteger = bigMultiplyPowerTen.add(longMultiplyPowerTen);
            }
        }
        else {
            bigMultiplyPowerTen = bigMultiplyPowerTen(bigMultiplyPowerTen, checkScale(bigMultiplyPowerTen, n3));
            bigInteger = bigMultiplyPowerTen.add(i);
        }
        return n4 ? new BigDecimal(bigInteger, Long.MIN_VALUE, n, 0) : valueOf(bigInteger, n, 0);
    }
    
    private static BigDecimal add(BigInteger bigMultiplyPowerTen, int n, BigInteger bigMultiplyPowerTen2, int n2) {
        long n3 = (long)n - (long)n2;
        if (n3 != 0L) {
            if (n3 < 0L) {
                int checkScale = checkScale(bigMultiplyPowerTen, -n3);
                n = n2;
                bigMultiplyPowerTen = bigMultiplyPowerTen(bigMultiplyPowerTen, checkScale);
            }
            else {
                bigMultiplyPowerTen2 = bigMultiplyPowerTen(bigMultiplyPowerTen2, checkScale(bigMultiplyPowerTen2, n3));
            }
        }
        BigInteger add = bigMultiplyPowerTen.add(bigMultiplyPowerTen2);
        return (bigMultiplyPowerTen.signum == bigMultiplyPowerTen2.signum) ? new BigDecimal(add, Long.MIN_VALUE, n, 0) : valueOf(add, n, 0);
    }
    
    private static BigInteger bigMultiplyPowerTen(long n, int n2) {
        if (n2 <= 0) {
            return BigInteger.valueOf(n);
        }
        return bigTenToThe(n2).multiply(n);
    }
    
    private static BigInteger bigMultiplyPowerTen(BigInteger bigInteger, int n) {
        if (n <= 0) {
            return bigInteger;
        }
        if (n < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
            return bigInteger.multiply(BigDecimal.LONG_TEN_POWERS_TABLE[n]);
        }
        return bigInteger.multiply(bigTenToThe(n));
    }
    
    private static BigDecimal divideSmallFastPath(long n, int n2, long n3, int n4, long n5, MathContext mathContext) {
        int precision = mathContext.precision;
        int oldMode = mathContext.roundingMode.oldMode;
        assert n2 <= n4 && n4 < 18 && precision < 18;
        int n6 = n4 - n2;
        long n7 = (n6 == 0) ? n : longMultiplyPowerTen(n, n6);
        int longCompareMagnitude = longCompareMagnitude(n7, n3);
        BigDecimal bigDecimal;
        if (longCompareMagnitude > 0) {
            --n4;
            int checkScaleNonZero2 = checkScaleNonZero(n5 + (long)n4 - (long)n2 + (long)precision);
            if (checkScaleNonZero((long)precision + (long)n4 - (long)n2) > 0) {
                long longMultiplyPowerTen3;
                if ((longMultiplyPowerTen3 = longMultiplyPowerTen(n, checkScaleNonZero((long)precision + (long)n4 - (long)n2))) == Long.MIN_VALUE) {
                    bigDecimal = null;
                    if (precision - 1 >= 0 && precision - 1 < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
                        bigDecimal = multiplyDivideAndRound(BigDecimal.LONG_TEN_POWERS_TABLE[precision - 1], n7, n3, checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                    }
                    if (bigDecimal == null) {
                        bigDecimal = divideAndRound(bigMultiplyPowerTen(n7, precision - 1), n3, checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                    }
                }
                else {
                    bigDecimal = divideAndRound(longMultiplyPowerTen3, n3, checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                }
            }
            else {
                int checkScaleNonZero3 = checkScaleNonZero((long)n2 - (long)precision);
                if (checkScaleNonZero3 == n4) {
                    bigDecimal = divideAndRound(n, n3, checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                }
                else {
                    int checkScaleNonZero4 = checkScaleNonZero((long)checkScaleNonZero3 - (long)n4);
                    long longMultiplyPowerTen2;
                    if ((longMultiplyPowerTen2 = longMultiplyPowerTen(n3, checkScaleNonZero4)) == Long.MIN_VALUE) {
                        bigDecimal = divideAndRound(BigInteger.valueOf(n), bigMultiplyPowerTen(n3, checkScaleNonZero4), checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                    }
                    else {
                        bigDecimal = divideAndRound(n, longMultiplyPowerTen2, checkScaleNonZero2, oldMode, checkScaleNonZero(n5));
                    }
                }
            }
        }
        else {
            int checkScaleNonZero = checkScaleNonZero(n5 + (long)n4 - (long)n2 + (long)precision);
            if (longCompareMagnitude == 0) {
                bigDecimal = roundedTenPower((n7 < 0L == n3 < 0L) ? 1 : -1, precision, checkScaleNonZero, checkScaleNonZero(n5));
            }
            else {
                long longMultiplyPowerTen;
                if ((longMultiplyPowerTen = longMultiplyPowerTen(n7, precision)) == Long.MIN_VALUE) {
                    bigDecimal = null;
                    if (precision < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
                        bigDecimal = multiplyDivideAndRound(BigDecimal.LONG_TEN_POWERS_TABLE[precision], n7, n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
                    }
                    if (bigDecimal == null) {
                        bigDecimal = divideAndRound(bigMultiplyPowerTen(n7, precision), n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
                    }
                }
                else {
                    bigDecimal = divideAndRound(longMultiplyPowerTen, n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
                }
            }
        }
        return doRound(bigDecimal, mathContext);
    }
    
    private static BigDecimal divide(long n, int n2, long n3, int n4, long n5, MathContext mathContext) {
        int precision = mathContext.precision;
        if (n2 <= n4 && n4 < 18 && precision < 18) {
            return divideSmallFastPath(n, n2, n3, n4, n5, mathContext);
        }
        if (compareMagnitudeNormalized(n, n2, n3, n4) > 0) {
            --n4;
        }
        int oldMode = mathContext.roundingMode.oldMode;
        int checkScaleNonZero = checkScaleNonZero(n5 + (long)n4 - (long)n2 + (long)precision);
        BigDecimal bigDecimal;
        if (checkScaleNonZero((long)precision + (long)n4 - (long)n2) > 0) {
            int checkScaleNonZero4 = checkScaleNonZero((long)precision + (long)n4 - (long)n2);
            long longMultiplyPowerTen2;
            if ((longMultiplyPowerTen2 = longMultiplyPowerTen(n, checkScaleNonZero4)) == Long.MIN_VALUE) {
                bigDecimal = divideAndRound(bigMultiplyPowerTen(n, checkScaleNonZero4), n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
            }
            else {
                bigDecimal = divideAndRound(longMultiplyPowerTen2, n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
            }
        }
        else {
            int checkScaleNonZero2 = checkScaleNonZero((long)n2 - (long)precision);
            if (checkScaleNonZero2 == n4) {
                bigDecimal = divideAndRound(n, n3, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
            }
            else {
                int checkScaleNonZero3 = checkScaleNonZero((long)checkScaleNonZero2 - (long)n4);
                long longMultiplyPowerTen;
                if ((longMultiplyPowerTen = longMultiplyPowerTen(n3, checkScaleNonZero3)) == Long.MIN_VALUE) {
                    bigDecimal = divideAndRound(BigInteger.valueOf(n), bigMultiplyPowerTen(n3, checkScaleNonZero3), checkScaleNonZero, oldMode, checkScaleNonZero(n5));
                }
                else {
                    bigDecimal = divideAndRound(n, longMultiplyPowerTen, checkScaleNonZero, oldMode, checkScaleNonZero(n5));
                }
            }
        }
        return doRound(bigDecimal, mathContext);
    }
    
    private static BigDecimal divide(BigInteger bigInteger, int n, long n2, int n3, long n4, MathContext mathContext) {
        if (-compareMagnitudeNormalized(n2, n3, bigInteger, n) > 0) {
            --n3;
        }
        int precision = mathContext.precision;
        int oldMode = mathContext.roundingMode.oldMode;
        int checkScaleNonZero = checkScaleNonZero(n4 + (long)n3 - (long)n + (long)precision);
        BigDecimal bigDecimal;
        if (checkScaleNonZero((long)precision + (long)n3 - (long)n) > 0) {
            bigDecimal = divideAndRound(bigMultiplyPowerTen(bigInteger, checkScaleNonZero((long)precision + (long)n3 - (long)n)), n2, checkScaleNonZero, oldMode, checkScaleNonZero(n4));
        }
        else {
            int checkScaleNonZero2 = checkScaleNonZero((long)n - (long)precision);
            if (checkScaleNonZero2 == n3) {
                bigDecimal = divideAndRound(bigInteger, n2, checkScaleNonZero, oldMode, checkScaleNonZero(n4));
            }
            else {
                int checkScaleNonZero3 = checkScaleNonZero((long)checkScaleNonZero2 - (long)n3);
                long longMultiplyPowerTen;
                if ((longMultiplyPowerTen = longMultiplyPowerTen(n2, checkScaleNonZero3)) == Long.MIN_VALUE) {
                    bigDecimal = divideAndRound(bigInteger, bigMultiplyPowerTen(n2, checkScaleNonZero3), checkScaleNonZero, oldMode, checkScaleNonZero(n4));
                }
                else {
                    bigDecimal = divideAndRound(bigInteger, longMultiplyPowerTen, checkScaleNonZero, oldMode, checkScaleNonZero(n4));
                }
            }
        }
        return doRound(bigDecimal, mathContext);
    }
    
    private static BigDecimal divide(long n, int n2, BigInteger bigInteger, int n3, long n4, MathContext mathContext) {
        if (compareMagnitudeNormalized(n, n2, bigInteger, n3) > 0) {
            --n3;
        }
        int precision = mathContext.precision;
        int oldMode = mathContext.roundingMode.oldMode;
        int checkScaleNonZero = checkScaleNonZero(n4 + (long)n3 - (long)n2 + (long)precision);
        BigDecimal bigDecimal;
        if (checkScaleNonZero((long)precision + (long)n3 - (long)n2) > 0) {
            bigDecimal = divideAndRound(bigMultiplyPowerTen(n, checkScaleNonZero((long)precision + (long)n3 - (long)n2)), bigInteger, checkScaleNonZero, oldMode, checkScaleNonZero(n4));
        }
        else {
            bigDecimal = divideAndRound(BigInteger.valueOf(n), bigMultiplyPowerTen(bigInteger, checkScaleNonZero((long)checkScaleNonZero((long)n2 - (long)precision) - (long)n3)), checkScaleNonZero, oldMode, checkScaleNonZero(n4));
        }
        return doRound(bigDecimal, mathContext);
    }
    
    private static BigDecimal divide(BigInteger bigInteger, int n, BigInteger bigInteger2, int n2, long n3, MathContext mathContext) {
        if (compareMagnitudeNormalized(bigInteger, n, bigInteger2, n2) > 0) {
            --n2;
        }
        int precision = mathContext.precision;
        int oldMode = mathContext.roundingMode.oldMode;
        int checkScaleNonZero = checkScaleNonZero(n3 + (long)n2 - (long)n + (long)precision);
        BigDecimal bigDecimal;
        if (checkScaleNonZero((long)precision + (long)n2 - (long)n) > 0) {
            bigDecimal = divideAndRound(bigMultiplyPowerTen(bigInteger, checkScaleNonZero((long)precision + (long)n2 - (long)n)), bigInteger2, checkScaleNonZero, oldMode, checkScaleNonZero(n3));
        }
        else {
            bigDecimal = divideAndRound(bigInteger, bigMultiplyPowerTen(bigInteger2, checkScaleNonZero((long)checkScaleNonZero((long)n - (long)precision) - (long)n2)), checkScaleNonZero, oldMode, checkScaleNonZero(n3));
        }
        return doRound(bigDecimal, mathContext);
    }
    
    private static BigDecimal multiplyDivideAndRound(long abs, long abs2, long abs3, int n, int n2, int n3) {
        int n4 = Long.signum(abs) * Long.signum(abs2) * Long.signum(abs3);
        abs = Math.abs(abs);
        abs2 = Math.abs(abs2);
        abs3 = Math.abs(abs3);
        long n5 = abs >>> 32;
        long n6 = abs & 0xFFFFFFFFL;
        long n7 = abs2 >>> 32;
        long n8 = abs2 & 0xFFFFFFFFL;
        long n9 = n6 * n8;
        long n10 = n9 & 0xFFFFFFFFL;
        long n11 = n5 * n8 + (n9 >>> 32);
        long n12 = n11 & 0xFFFFFFFFL;
        long n13 = n11 >>> 32;
        long n14 = n6 * n7 + n12;
        long n15 = n14 & 0xFFFFFFFFL;
        long n16 = n13 + (n14 >>> 32);
        long n17 = n16 >>> 32;
        long n18 = n5 * n7 + (n16 & 0xFFFFFFFFL);
        return divideAndRound128(make64((n18 >>> 32) + n17 & 0xFFFFFFFFL, n18 & 0xFFFFFFFFL), make64(n15, n10), abs3, n4, n, n2, n3);
    }
    
    private static BigDecimal divideAndRound128(long n, long n2, long i, int n3, int n4, int n5, int n6) {
        if (n >= i) {
            return null;
        }
        int numberOfLeadingZeros = Long.numberOfLeadingZeros(i);
        i <<= numberOfLeadingZeros;
        long n7 = i >>> 32;
        long n8 = i & 0xFFFFFFFFL;
        long n9 = n2 << numberOfLeadingZeros;
        long n10 = n9 >>> 32;
        long n11 = n9 & 0xFFFFFFFFL;
        long n12 = n << numberOfLeadingZeros | n2 >>> 64 - numberOfLeadingZeros;
        long n13 = n12 & 0xFFFFFFFFL;
        long n14;
        if (n7 == 1L) {
            n14 = 0L;
        }
        else if (n12 >= 0L) {
            n12 /= n7;
            n14 = n12 - n12 * n7;
        }
        else {
            long[] divRemNegativeLong = divRemNegativeLong(n12, n7);
            n12 = divRemNegativeLong[1];
            n14 = divRemNegativeLong[0];
        }
        while (n12 >= 4294967296L || unsignedLongCompare(n12 * n8, make64(n14, n10))) {
            --n12;
            n14 += n7;
            if (n14 >= 4294967296L) {
                break;
            }
            continue;
        }
        long mulsub = mulsub(n13, n10, n7, n8, n12);
        long n15 = mulsub & 0xFFFFFFFFL;
        long n16;
        if (n7 == 1L) {
            n16 = 0L;
        }
        else if (mulsub >= 0L) {
            mulsub /= n7;
            n16 = mulsub - mulsub * n7;
        }
        else {
            long[] divRemNegativeLong2 = divRemNegativeLong(mulsub, n7);
            mulsub = divRemNegativeLong2[1];
            n16 = divRemNegativeLong2[0];
        }
        while (mulsub >= 4294967296L || unsignedLongCompare(mulsub * n8, make64(n16, n11))) {
            --mulsub;
            n16 += n7;
            if (n16 >= 4294967296L) {
                break;
            }
            continue;
        }
        if ((int)n12 < 0) {
            MutableBigInteger o = new MutableBigInteger(new int[] { (int)n12, (int)mulsub });
            if (n5 == 1 && n4 == n6) {
                return o.toBigDecimal(n3, n4);
            }
            long n19 = mulsub(n15, n11, n7, n8, mulsub) >>> numberOfLeadingZeros;
            if (n19 != 0L) {
                if (needIncrement(i >>> numberOfLeadingZeros, n5, n3, o, n19)) {
                    o.add(MutableBigInteger.ONE);
                }
                return o.toBigDecimal(n3, n4);
            }
            else {
                if (n6 != n4) {
                    return createAndStripZerosToMatchScale(o.toBigInteger(n3), n4, (long)n6);
                }
                return o.toBigDecimal(n3, n4);
            }
        }
        else {
            long n17 = make64(n12, mulsub) * (long)n3;
            if (n5 == 1 && n4 == n6) {
                return valueOf(n17, n4);
            }
            long n18 = mulsub(n15, n11, n7, n8, mulsub) >>> numberOfLeadingZeros;
            if (n18 != 0L) {
                return valueOf(needIncrement(i >>> numberOfLeadingZeros, n5, n3, n17, n18) ? (n17 + (long)n3) : n17, n4);
            }
            if (n6 != n4) {
                return createAndStripZerosToMatchScale(n17, n4, (long)n6);
            }
            return valueOf(n17, n4);
        }
    }
    
    private static BigDecimal roundedTenPower(int n, int n2, int n3, int n4) {
        if (n3 > n4) {
            int n5 = n3 - n4;
            if (n5 < n2) {
                return scaledTenPow(n2 - n5, n, n4);
            }
            return valueOf((long)n, n3 - n2);
        }
        else {
            return scaledTenPow(n2, n, n3);
        }
    }
    
    static BigDecimal scaledTenPow(int n, int n2, int n3) {
        if (n < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
            return valueOf((long)n2 * BigDecimal.LONG_TEN_POWERS_TABLE[n], n3);
        }
        BigInteger bigInteger = bigTenToThe(n);
        if (n2 == -1) {
            bigInteger = bigInteger.negate();
        }
        return new BigDecimal(bigInteger, Long.MIN_VALUE, n3, n + 1);
    }
    
    private static long[] divRemNegativeLong(long lng, long n) {
        assert lng < 0L : "Non-negative numerator " + lng;
        assert n != 1L : "Unity denominator";
        long n2;
        long n3;
        for (n2 = (lng >>> 1) / (n >>> 1), n3 = lng - n2 * n; n3 < 0L; n3 += n, --n2) {}
        while (n3 >= n) {
            n3 -= n;
            ++n2;
        }
        return new long[] { n3, n2 };
    }
    
    private static long make64(long n, long n2) {
        return n << 32 | n2;
    }
    
    private static long mulsub(long n, long n2, long n3, long n4, long n5) {
        long n6 = n2 - n5 * n4;
        return make64(n + (n6 >>> 32) - n5 * n3, n6 & 0xFFFFFFFFL);
    }
    
    private static boolean unsignedLongCompare(long n, long n2) {
        return n + Long.MIN_VALUE > n2 + Long.MIN_VALUE;
    }
    
    private static boolean unsignedLongCompareEq(long n, long n2) {
        return n + Long.MIN_VALUE >= n2 + Long.MIN_VALUE;
    }
    
    private static int compareMagnitudeNormalized(long longMultiplyPowerTen, int n, long longMultiplyPowerTen2, int n2) {
        int n3 = n - n2;
        if (n3 != 0) {
            if (n3 < 0) {
                longMultiplyPowerTen = longMultiplyPowerTen(longMultiplyPowerTen, -n3);
            }
            else {
                longMultiplyPowerTen2 = longMultiplyPowerTen(longMultiplyPowerTen2, n3);
            }
        }
        if (longMultiplyPowerTen != Long.MIN_VALUE) {
            return (longMultiplyPowerTen2 != Long.MIN_VALUE) ? longCompareMagnitude(longMultiplyPowerTen, longMultiplyPowerTen2) : -1;
        }
        return 1;
    }
    
    private static int compareMagnitudeNormalized(long n, int n2, BigInteger bigInteger, int n3) {
        if (n == 0L) {
            return -1;
        }
        int n4 = n2 - n3;
        if (n4 < 0 && longMultiplyPowerTen(n, -n4) == Long.MIN_VALUE) {
            return bigMultiplyPowerTen(n, -n4).compareMagnitude(bigInteger);
        }
        return -1;
    }
    
    private static int compareMagnitudeNormalized(BigInteger bigInteger, int n, BigInteger bigInteger2, int n2) {
        int n3 = n - n2;
        if (n3 < 0) {
            return bigMultiplyPowerTen(bigInteger, -n3).compareMagnitude(bigInteger2);
        }
        return bigInteger.compareMagnitude(bigMultiplyPowerTen(bigInteger2, n3));
    }
    
    private static long multiply(long a, long a2) {
        long n = a * a2;
        if ((Math.abs(a) | Math.abs(a2)) >>> 31 == 0L || a2 == 0L || n / a2 == a) {
            return n;
        }
        return Long.MIN_VALUE;
    }
    
    private static BigDecimal multiply(long n, long n2, int n3) {
        long multiply = multiply(n, n2);
        if (multiply != Long.MIN_VALUE) {
            return valueOf(multiply, n3);
        }
        return new BigDecimal(BigInteger.valueOf(n).multiply(n2), Long.MIN_VALUE, n3, 0);
    }
    
    private static BigDecimal multiply(long n, BigInteger bigInteger, int n2) {
        if (n == 0L) {
            return zeroValueOf(n2);
        }
        return new BigDecimal(bigInteger.multiply(n), Long.MIN_VALUE, n2, 0);
    }
    
    private static BigDecimal multiply(BigInteger bigInteger, BigInteger bigInteger2, int n) {
        return new BigDecimal(bigInteger.multiply(bigInteger2), Long.MIN_VALUE, n, 0);
    }
    
    private static BigDecimal multiplyAndRound(long n, long n2, int n3, MathContext mathContext) {
        long multiply = multiply(n, n2);
        if (multiply != Long.MIN_VALUE) {
            return doRound(multiply, n3, mathContext);
        }
        int n4 = 1;
        if (n < 0L) {
            n = -n;
            n4 = -1;
        }
        if (n2 < 0L) {
            n2 = -n2;
            n4 *= -1;
        }
        long n5 = n >>> 32;
        long n6 = n & 0xFFFFFFFFL;
        long n7 = n2 >>> 32;
        long n8 = n2 & 0xFFFFFFFFL;
        long n9 = n6 * n8;
        long n10 = n9 & 0xFFFFFFFFL;
        long n11 = n5 * n8 + (n9 >>> 32);
        long n12 = n11 & 0xFFFFFFFFL;
        long n13 = n11 >>> 32;
        long n14 = n6 * n7 + n12;
        long n15 = n14 & 0xFFFFFFFFL;
        long n16 = n13 + (n14 >>> 32);
        long n17 = n16 >>> 32;
        long n18 = n5 * n7 + (n16 & 0xFFFFFFFFL);
        BigDecimal doRound128 = doRound128(make64((n18 >>> 32) + n17 & 0xFFFFFFFFL, n18 & 0xFFFFFFFFL), make64(n15, n10), n4, n3, mathContext);
        if (doRound128 != null) {
            return doRound128;
        }
        return doRound(new BigDecimal(BigInteger.valueOf(n).multiply(n2 * (long)n4), Long.MIN_VALUE, n3, 0), mathContext);
    }
    
    private static BigDecimal multiplyAndRound(long n, BigInteger bigInteger, int n2, MathContext mathContext) {
        if (n == 0L) {
            return zeroValueOf(n2);
        }
        return doRound(bigInteger.multiply(n), n2, mathContext);
    }
    
    private static BigDecimal multiplyAndRound(BigInteger bigInteger, BigInteger bigInteger2, int n, MathContext mathContext) {
        return doRound(bigInteger.multiply(bigInteger2), n, mathContext);
    }
    
    private static BigDecimal doRound128(long n, long n2, int n3, int checkScaleNonZero, MathContext mathContext) {
        int precision = mathContext.precision;
        BigDecimal divideAndRound128 = null;
        int n4;
        if ((n4 = precision(n, n2) - precision) > 0 && n4 < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
            checkScaleNonZero = checkScaleNonZero((long)checkScaleNonZero - (long)n4);
            divideAndRound128 = divideAndRound128(n, n2, BigDecimal.LONG_TEN_POWERS_TABLE[n4], n3, checkScaleNonZero, mathContext.roundingMode.oldMode, checkScaleNonZero);
        }
        if (divideAndRound128 != null) {
            return doRound(divideAndRound128, mathContext);
        }
        return null;
    }
    
    private static int precision(long i, long n) {
        if (i == 0L) {
            if (n >= 0L) {
                return longDigitLength(n);
            }
            return unsignedLongCompareEq(n, BigDecimal.LONGLONG_TEN_POWERS_TABLE[0][1]) ? 20 : 19;
        }
        else {
            int n2 = (128 - Long.numberOfLeadingZeros(i) + 1) * 1233 >>> 12;
            int n3 = n2 - 19;
            return (n3 >= BigDecimal.LONGLONG_TEN_POWERS_TABLE.length || longLongCompareMagnitude(i, n, BigDecimal.LONGLONG_TEN_POWERS_TABLE[n3][0], BigDecimal.LONGLONG_TEN_POWERS_TABLE[n3][1])) ? n2 : (n2 + 1);
        }
    }
    
    private static boolean longLongCompareMagnitude(long n, long n2, long n3, long n4) {
        if (n != n3) {
            return n < n3;
        }
        return n2 + Long.MIN_VALUE < n4 + Long.MIN_VALUE;
    }
    
    private static BigDecimal divide(long n, int n2, long n3, int n4, int n5, int n6) {
        if (checkScale(n, (long)n5 + (long)n4) > n2) {
            int n8 = n5 + n4 - n2;
            if (n8 < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
                long longMultiplyPowerTen2;
                if ((longMultiplyPowerTen2 = longMultiplyPowerTen(n, n8)) != Long.MIN_VALUE) {
                    return divideAndRound(longMultiplyPowerTen2, n3, n5, n6, n5);
                }
                BigDecimal multiplyDivideAndRound = multiplyDivideAndRound(BigDecimal.LONG_TEN_POWERS_TABLE[n8], n, n3, n5, n6, n5);
                if (multiplyDivideAndRound != null) {
                    return multiplyDivideAndRound;
                }
            }
            return divideAndRound(bigMultiplyPowerTen(n, n8), n3, n5, n6, n5);
        }
        else {
            int n7 = checkScale(n3, (long)n2 - (long)n5) - n4;
            long longMultiplyPowerTen;
            if (n7 < BigDecimal.LONG_TEN_POWERS_TABLE.length && (longMultiplyPowerTen = longMultiplyPowerTen(n3, n7)) != Long.MIN_VALUE) {
                return divideAndRound(n, longMultiplyPowerTen, n5, n6, n5);
            }
            return divideAndRound(BigInteger.valueOf(n), bigMultiplyPowerTen(n3, n7), n5, n6, n5);
        }
    }
    
    private static BigDecimal divide(BigInteger bigInteger, int n, long n2, int n3, int n4, int n5) {
        if (checkScale(bigInteger, (long)n4 + (long)n3) > n) {
            return divideAndRound(bigMultiplyPowerTen(bigInteger, n4 + n3 - n), n2, n4, n5, n4);
        }
        int n6 = checkScale(n2, (long)n - (long)n4) - n3;
        if (n6 < BigDecimal.LONG_TEN_POWERS_TABLE.length) {
            long longMultiplyPowerTen;
            if ((longMultiplyPowerTen = longMultiplyPowerTen(n2, n6)) != Long.MIN_VALUE) {
                return divideAndRound(bigInteger, longMultiplyPowerTen, n4, n5, n4);
            }
        }
        return divideAndRound(bigInteger, bigMultiplyPowerTen(n2, n6), n4, n5, n4);
    }
    
    private static BigDecimal divide(long n, int n2, BigInteger bigInteger, int n3, int n4, int n5) {
        if (checkScale(n, (long)n4 + (long)n3) > n2) {
            return divideAndRound(bigMultiplyPowerTen(n, n4 + n3 - n2), bigInteger, n4, n5, n4);
        }
        return divideAndRound(BigInteger.valueOf(n), bigMultiplyPowerTen(bigInteger, checkScale(bigInteger, (long)n2 - (long)n4) - n3), n4, n5, n4);
    }
    
    private static BigDecimal divide(BigInteger bigInteger, int n, BigInteger bigInteger2, int n2, int n3, int n4) {
        if (checkScale(bigInteger, (long)n3 + (long)n2) > n) {
            return divideAndRound(bigMultiplyPowerTen(bigInteger, n3 + n2 - n), bigInteger2, n3, n4, n3);
        }
        return divideAndRound(bigInteger, bigMultiplyPowerTen(bigInteger2, checkScale(bigInteger2, (long)n - (long)n3) - n2), n3, n4, n3);
    }
    
    static BigInteger access$000(BigDecimal bigDecimal) {
        return bigDecimal.inflated();
    }
    
    static {
        INFLATED_BIGINT = BigInteger.valueOf(Long.MIN_VALUE);
        threadLocalStringBuilderHelper = new ThreadLocal<BigDecimal.StringBuilderHelper>() {
            @Override
            protected BigDecimal.StringBuilderHelper initialValue() {
                return new BigDecimal.StringBuilderHelper();
            }
        };
        zeroThroughTen = new BigDecimal[] { new BigDecimal(BigInteger.ZERO, 0L, 0, 1), new BigDecimal(BigInteger.ONE, 1L, 0, 1), new BigDecimal(BigInteger.valueOf(2L), 2L, 0, 1), new BigDecimal(BigInteger.valueOf(3L), 3L, 0, 1), new BigDecimal(BigInteger.valueOf(4L), 4L, 0, 1), new BigDecimal(BigInteger.valueOf(5L), 5L, 0, 1), new BigDecimal(BigInteger.valueOf(6L), 6L, 0, 1), new BigDecimal(BigInteger.valueOf(7L), 7L, 0, 1), new BigDecimal(BigInteger.valueOf(8L), 8L, 0, 1), new BigDecimal(BigInteger.valueOf(9L), 9L, 0, 1), new BigDecimal(BigInteger.TEN, 10L, 0, 2) };
        ZERO_SCALED_BY = new BigDecimal[] { BigDecimal.zeroThroughTen[0], new BigDecimal(BigInteger.ZERO, 0L, 1, 1), new BigDecimal(BigInteger.ZERO, 0L, 2, 1), new BigDecimal(BigInteger.ZERO, 0L, 3, 1), new BigDecimal(BigInteger.ZERO, 0L, 4, 1), new BigDecimal(BigInteger.ZERO, 0L, 5, 1), new BigDecimal(BigInteger.ZERO, 0L, 6, 1), new BigDecimal(BigInteger.ZERO, 0L, 7, 1), new BigDecimal(BigInteger.ZERO, 0L, 8, 1), new BigDecimal(BigInteger.ZERO, 0L, 9, 1), new BigDecimal(BigInteger.ZERO, 0L, 10, 1), new BigDecimal(BigInteger.ZERO, 0L, 11, 1), new BigDecimal(BigInteger.ZERO, 0L, 12, 1), new BigDecimal(BigInteger.ZERO, 0L, 13, 1), new BigDecimal(BigInteger.ZERO, 0L, 14, 1), new BigDecimal(BigInteger.ZERO, 0L, 15, 1) };
        ZERO = BigDecimal.zeroThroughTen[0];
        ONE = BigDecimal.zeroThroughTen[1];
        TEN = BigDecimal.zeroThroughTen[10];
        double10pow = new double[] { 1.0, 10.0, 100.0, 1000.0, 10000.0, 100000.0, 1000000.0, 1.0E7, 1.0E8, 1.0E9, 1.0E10, 1.0E11, 1.0E12, 1.0E13, 1.0E14, 1.0E15, 1.0E16, 1.0E17, 1.0E18, 1.0E19, 1.0E20, 1.0E21, 1.0E22 };
        float10pow = new float[] { 1.0f, 10.0f, 100.0f, 1000.0f, 10000.0f, 100000.0f, 1000000.0f, 1.0E7f, 1.0E8f, 1.0E9f, 1.0E10f };
        LONG_TEN_POWERS_TABLE = new long[] { 1L, 10L, 100L, 1000L, 10000L, 100000L, 1000000L, 10000000L, 100000000L, 1000000000L, 10000000000L, 100000000000L, 1000000000000L, 10000000000000L, 100000000000000L, 1000000000000000L, 10000000000000000L, 100000000000000000L, 1000000000000000000L };
        BigDecimal.BIG_TEN_POWERS_TABLE = new BigInteger[] { BigInteger.ONE, BigInteger.valueOf(10L), BigInteger.valueOf(100L), BigInteger.valueOf(1000L), BigInteger.valueOf(10000L), BigInteger.valueOf(100000L), BigInteger.valueOf(1000000L), BigInteger.valueOf(10000000L), BigInteger.valueOf(100000000L), BigInteger.valueOf(1000000000L), BigInteger.valueOf(10000000000L), BigInteger.valueOf(100000000000L), BigInteger.valueOf(1000000000000L), BigInteger.valueOf(10000000000000L), BigInteger.valueOf(100000000000000L), BigInteger.valueOf(1000000000000000L), BigInteger.valueOf(10000000000000000L), BigInteger.valueOf(100000000000000000L), BigInteger.valueOf(1000000000000000000L) };
        BIG_TEN_POWERS_TABLE_INITLEN = BigDecimal.BIG_TEN_POWERS_TABLE.length;
        BIG_TEN_POWERS_TABLE_MAX = 16 * BigDecimal.BIG_TEN_POWERS_TABLE_INITLEN;
        THRESHOLDS_TABLE = new long[] { Long.MAX_VALUE, 922337203685477580L, 92233720368547758L, 9223372036854775L, 922337203685477L, 92233720368547L, 9223372036854L, 922337203685L, 92233720368L, 9223372036L, 922337203L, 92233720L, 9223372L, 922337L, 92233L, 9223L, 922L, 92L, 9L };
        LONGLONG_TEN_POWERS_TABLE = new long[][] { { 0L, -8446744073709551616L }, { 5L, 7766279631452241920L }, { 54L, 3875820019684212736L }, { 542L, 1864712049423024128L }, { 5421L, 200376420520689664L }, { 54210L, 2003764205206896640L }, { 542101L, 1590897978359414784L }, { 5421010L, -2537764290115403776L }, { 54210108L, -6930898827444486144L }, { 542101086L, 4477988020393345024L }, { 5421010862L, 7886392056514347008L }, { 54210108624L, 5076944270305263616L }, { 542101086242L, -4570789518076018688L }, { 5421010862427L, -8814407033341083648L }, { 54210108624275L, 4089650035136921600L }, { 542101086242752L, 4003012203950112768L }, { 5421010862427522L, 3136633892082024448L }, { 54210108624275221L, -5527149226598858752L }, { 542101086242752217L, 68739955140067328L }, { 5421010862427522170L, 687399551400673280L } };
    }
    
    private static class LongOverflow
    {
        private static final BigInteger LONGMIN;
        private static final BigInteger LONGMAX;
        
        public static void check(BigDecimal bigDecimal) {
            BigInteger access$000 = BigDecimal.access$000(bigDecimal);
            if (access$000.compareTo(BigDecimal.LongOverflow.LONGMIN) < 0 || access$000.compareTo(BigDecimal.LongOverflow.LONGMAX) > 0) {
                throw new ArithmeticException("Overflow");
            }
            return;
        }
        
        static {
            LONGMIN = BigInteger.valueOf(Long.MIN_VALUE);
            LONGMAX = BigInteger.valueOf(Long.MAX_VALUE);
        }
    }
    
    static class StringBuilderHelper
    {
        final StringBuilder sb;
        final char[] cmpCharArray;
        static final char[] DIGIT_TENS;
        static final char[] DIGIT_ONES;
        
        StringBuilderHelper() {
            this.sb = new StringBuilder();
            this.cmpCharArray = new char[19];
        }
        
        StringBuilder getStringBuilder() {
            this.sb.setLength(0);
            return this.sb;
        }
        
        char[] getCompactCharArray() {
            return this.cmpCharArray;
        }
        
        int putIntCompact(long n) {
            assert n >= 0L;
            int length;
            long n2;
            int n3;
            for (length = this.cmpCharArray.length; n > 2147483647L; n = n2, this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_ONES[n3], this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_TENS[n3]) {
                n2 = n / 100L;
                n3 = (int)(n - n2 * 100L);
            }
            int i;
            int n4;
            int n5;
            for (i = (int)n; i >= 100; i = n4, this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_ONES[n5], this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_TENS[n5]) {
                n4 = i / 100;
                n5 = i - n4 * 100;
            }
            this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_ONES[i];
            if (i >= 10) {
                this.cmpCharArray[--length] = BigDecimal.StringBuilderHelper.DIGIT_TENS[i];
            }
            return length;
        }
        
        static {
            DIGIT_TENS = new char[] { '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '1', '1', '1', '1', '1', '1', '1', '1', '1', '1', '2', '2', '2', '2', '2', '2', '2', '2', '2', '2', '3', '3', '3', '3', '3', '3', '3', '3', '3', '3', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '5', '5', '5', '5', '5', '5', '5', '5', '5', '5', '6', '6', '6', '6', '6', '6', '6', '6', '6', '6', '7', '7', '7', '7', '7', '7', '7', '7', '7', '7', '8', '8', '8', '8', '8', '8', '8', '8', '8', '8', '9', '9', '9', '9', '9', '9', '9', '9', '9', '9' };
            DIGIT_ONES = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
        }
    }
    
    private static class UnsafeHolder
    {
        private static final Unsafe unsafe;
        private static final long intCompactOffset;
        private static final long intValOffset;
        
        static void setIntCompactVolatile(BigDecimal o, long x) {
            BigDecimal.UnsafeHolder.unsafe.putLongVolatile(o, BigDecimal.UnsafeHolder.intCompactOffset, x);
        }
        
        static void setIntValVolatile(BigDecimal o, BigInteger x) {
            BigDecimal.UnsafeHolder.unsafe.putObjectVolatile(o, BigDecimal.UnsafeHolder.intValOffset, x);
        }
        
        static {
            try {
                unsafe = Unsafe.getUnsafe();
                intCompactOffset = BigDecimal.UnsafeHolder.unsafe.objectFieldOffset(BigDecimal.class.getDeclaredField("intCompact"));
                intValOffset = BigDecimal.UnsafeHolder.unsafe.objectFieldOffset(BigDecimal.class.getDeclaredField("intVal"));
            }
            catch (Exception thrown) {
                throw new ExceptionInInitializerError(thrown);
            }
        }
    }
}
